import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipUnlessDarwin
    @no_debug_info_test
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.m")
        )

        # Test several variables with C++ keyword names and make sure they
        # work as intended in the expression parser.
        self.expect_expr("alignas", result_type="int", result_value="1")
        self.expect_expr("alignof", result_type="int", result_value="1")
        self.expect_expr("and", result_type="int", result_value="1")
        self.expect_expr("and_eq", result_type="int", result_value="1")
        self.expect_expr("atomic_cancel", result_type="int", result_value="1")
        self.expect_expr("atomic_commit", result_type="int", result_value="1")
        self.expect_expr("atomic_noexcept", result_type="int", result_value="1")
        self.expect_expr("bitand", result_type="int", result_value="1")
        self.expect_expr("bitor", result_type="int", result_value="1")
        self.expect_expr("catch", result_type="int", result_value="1")
        self.expect_expr("char8_t", result_type="int", result_value="1")
        self.expect_expr("char16_t", result_type="int", result_value="1")
        self.expect_expr("char32_t", result_type="int", result_value="1")
        self.expect_expr("class", result_type="int", result_value="1")
        self.expect_expr("compl", result_type="int", result_value="1")
        self.expect_expr("concept", result_type="int", result_value="1")
        self.expect_expr("consteval", result_type="int", result_value="1")
        self.expect_expr("constexpr", result_type="int", result_value="1")
        self.expect_expr("constinit", result_type="int", result_value="1")
        self.expect_expr("const_cast", result_type="int", result_value="1")
        self.expect_expr("co_await", result_type="int", result_value="1")
        self.expect_expr("co_return", result_type="int", result_value="1")
        self.expect_expr("co_yield", result_type="int", result_value="1")
        self.expect_expr("decltype", result_type="int", result_value="1")
        self.expect_expr("delete", result_type="int", result_value="1")
        self.expect_expr("dynamic_cast", result_type="int", result_value="1")
        self.expect_expr("explicit", result_type="int", result_value="1")
        self.expect_expr("export", result_type="int", result_value="1")
        self.expect_expr("friend", result_type="int", result_value="1")
        self.expect_expr("mutable", result_type="int", result_value="1")
        self.expect_expr("namespace", result_type="int", result_value="1")
        self.expect_expr("new", result_type="int", result_value="1")
        self.expect_expr("noexcept", result_type="int", result_value="1")
        self.expect_expr("not", result_type="int", result_value="1")
        self.expect_expr("not_eq", result_type="int", result_value="1")
        self.expect_expr("operator", result_type="int", result_value="1")
        self.expect_expr("or", result_type="int", result_value="1")
        self.expect_expr("or_eq", result_type="int", result_value="1")
        self.expect_expr("private", result_type="int", result_value="1")
        self.expect_expr("protected", result_type="int", result_value="1")
        self.expect_expr("public", result_type="int", result_value="1")
        self.expect_expr("reflexpr", result_type="int", result_value="1")
        self.expect_expr("reinterpret_cast", result_type="int", result_value="1")
        self.expect_expr("requires", result_type="int", result_value="1")
        self.expect_expr("static_assert", result_type="int", result_value="1")
        self.expect_expr("static_cast", result_type="int", result_value="1")
        self.expect_expr("synchronized", result_type="int", result_value="1")
        self.expect_expr("template", result_type="int", result_value="1")
        self.expect_expr("this", result_type="int", result_value="1")
        self.expect_expr("thread_local", result_type="int", result_value="1")
        self.expect_expr("throw", result_type="int", result_value="1")
        self.expect_expr("try", result_type="int", result_value="1")
        self.expect_expr("typeid", result_type="int", result_value="1")
        self.expect_expr("typename", result_type="int", result_value="1")
        self.expect_expr("virtual", result_type="int", result_value="1")
        self.expect_expr("xor", result_type="int", result_value="1")
        self.expect_expr("xor_eq", result_type="int", result_value="1")

        # Some keywords are not available in LLDB as their language feature
        # is enabled by default.

        # 'using' is used by LLDB for local variables.
        self.expect("expr using", error=True, substrs=["expected unqualified-id"])

        # 'wchar_t' supported is enabled in LLDB.
        self.expect("expr wchar_t", error=True, substrs=["expected unqualified-id"])

        # LLDB enables 'bool' support by default.
        self.expect("expr bool", error=True, substrs=["expected unqualified-id"])
        self.expect("expr false", error=True, substrs=["expected unqualified-id"])
        self.expect("expr true", error=True, substrs=["expected unqualified-id"])



"""
Test lldb-dap RestartRequest.
"""

from typing import Dict, Any, List

import lldbdap_testcase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import line_number


@skipIfBuildType(["debug"])
class TestDAP_restart_console(lldbdap_testcase.DAPTestCaseBase):
    @skipIfAsan
    @expectedFailureWindows
    @skipIf(oslist=["linux"], archs=["arm$"])  # Always times out on buildbot
    def test_basic_functionality(self):
        """
        Test basic restarting functionality when the process is running in
        a terminal.
        """
        line_A = line_number("main.c", "// breakpoint A")
        line_B = line_number("main.c", "// breakpoint B")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, console="integratedTerminal")
        [bp_A, bp_B] = self.set_source_breakpoints("main.c", [line_A, line_B])

        # Verify we hit A, then B.
        self.dap_server.request_configurationDone()
        self.verify_breakpoint_hit([bp_A])
        self.dap_server.request_continue()
        self.verify_breakpoint_hit([bp_B])

        # Make sure i has been modified from its initial value of 0.
        self.assertEqual(
            int(self.dap_server.get_local_variable_value("i")),
            1234,
            "i != 1234 after hitting breakpoint B",
        )

        # Restart.
        self.dap_server.request_restart()

        # Finally, check we stop back at A and program state has been reset.
        self.verify_breakpoint_hit([bp_A])
        self.assertEqual(
            int(self.dap_server.get_local_variable_value("i")),
            0,
            "i != 0 after hitting breakpoint A on restart",
        )

        # Check breakpoint B
        self.dap_server.request_continue()
        self.verify_breakpoint_hit([bp_B])
        self.assertEqual(
            int(self.dap_server.get_local_variable_value("i")),
            1234,
            "i != 1234 after hitting breakpoint B",
        )
        self.continue_to_exit()

    @skipIfAsan
    @expectedFailureWindows
    @skipIf(oslist=["linux"], archs=["arm$"])  # Always times out on buildbot
    def test_stopOnEntry(self):
        """
        Check that stopOnEntry works correctly when using console.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, console="integratedTerminal", stopOnEntry=True)
        [bp_main] = self.set_function_breakpoints(["main"])

        self.verify_configuration_done()
        self.verify_stop_on_entry()

        # Then, if we continue, we should hit the breakpoint at main.
        self.continue_to_breakpoints([bp_main])

        # Restart and check that we still get a stopped event before reaching
        # main.
        self.dap_server.request_restart()
        self.verify_stop_on_entry()

        # continue to main
        self.continue_to_breakpoints([bp_main])

        self.continue_to_exit()



"""
Test that an alias can contain active backticks
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestBackticksInAlias(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_backticks_in_alias(self):
        """Test that an alias can contain active backticks."""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()
        interp.HandleCommand(
            r"command alias _test-argv-cmd expression -Z \`argc\` -- argv", result
        )
        self.assertCommandReturn(result, "Made the alias")
        interp.HandleCommand("_test-argv-cmd", result)
        self.assertCommandReturn(result, "The alias worked")

        # Now try a harder case where we create this using an alias:
        interp.HandleCommand(
            r"command alias _test-argv-parray-cmd parray \`argc\` argv", result
        )
        self.assertCommandReturn(result, "Made the alias")
        interp.HandleCommand("_test-argv-parray-cmd", result)
        self.assertFalse(
            result.Succeeded(),
            "CommandAlias::Desugar currently fails if a alias substitutes %N arguments in another alias",
        )

    def test_backticks_in_parsed_cmd_argument(self):
        """break list is a parsed command, use a variable for the breakpoint number
        and make sure that and the direct use of the ID get the same result."""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        # Make a second breakpoint so that if the backtick part -> nothing we'll print too much:
        # It doesn't need to resolve to anything.
        dummy_bkpt = target.BreakpointCreateByName("dont_really_care_if_this_exists")

        bkpt_id = bkpt.GetID()
        self.runCmd(f"expr int $number = {bkpt_id}")
        direct_result = lldb.SBCommandReturnObject()
        backtick_result = lldb.SBCommandReturnObject()
        interp = self.dbg.GetCommandInterpreter()
        interp.HandleCommand(f"break list {bkpt_id}", direct_result)
        self.assertTrue(direct_result.Succeeded(), "Break list with id works")
        interp.HandleCommand("break list `$number`", backtick_result)
        self.assertTrue(direct_result.Succeeded(), "Break list with backtick works")
        self.assertEqual(
            direct_result.GetOutput(), backtick_result.GetOutput(), "Output is the same"
        )

    def test_backticks_in_parsed_cmd_option(self):
        # The script interpreter is a raw command, so try that one:
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )

        self.runCmd(f"expr int $number = 2")
        direct_result = lldb.SBCommandReturnObject()
        backtick_result = lldb.SBCommandReturnObject()
        interp = self.dbg.GetCommandInterpreter()
        interp.HandleCommand(f"memory read --count 2 argv", direct_result)
        self.assertTrue(
            direct_result.Succeeded(), "memory read with direct count works"
        )
        interp.HandleCommand("memory read --count `$number` argv", backtick_result)
        self.assertTrue(direct_result.Succeeded(), "memory read with backtick works")
        self.assertEqual(
            direct_result.GetOutput(), backtick_result.GetOutput(), "Output is the same"
        )

    def test_backticks_in_raw_cmd(self):
        # The script interpreter is a raw command, so try that one:
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        argc_valobj = thread.frames[0].FindVariable("argc")
        self.assertTrue(argc_valobj.GetError().Success(), "Made argc valobj")
        argc_value = argc_valobj.GetValueAsUnsigned(0)
        self.assertNotEqual(argc_value, 0, "Got a value for argc")
        result = lldb.SBCommandReturnObject()
        interp = self.dbg.GetCommandInterpreter()
        interp.HandleCommand(f"script {argc_value} - `argc`", result)
        self.assertTrue(result.Succeeded(), "Command succeeded")
        fixed_output = result.GetOutput().rstrip()
        self.assertEqual("0", fixed_output, "Substitution worked")



"""
Test lldb-dap stack trace when some of the source paths are missing
"""

from lldbsuite.test.lldbtest import line_number
import lldbdap_testcase
from contextlib import contextmanager
import os


OTHER_C_SOURCE_CODE = """
int no_source_func(int n) {
    return n + 1; // Break here
}
"""


@contextmanager
def delete_file_on_exit(path):
    try:
        yield path
    finally:
        if os.path.exists(path):
            os.remove(path)


class TestDAP_stackTraceMissingSourcePath(lldbdap_testcase.DAPTestCaseBase):
    def build_and_run_until_breakpoint(self):
        """
        Build the program and run until the breakpoint is hit, and return the stack frames.
        """
        other_source_file = self.getBuildArtifact("other.c")
        with delete_file_on_exit(other_source_file):
            with open(other_source_file, "w") as f:
                f.write(OTHER_C_SOURCE_CODE)

            breakpoint_line = line_number(other_source_file, "// Break here")

            program = self.getBuildArtifact("a.out")
            self.build_and_launch(program, commandEscapePrefix="")

            breakpoint_ids = self.set_source_breakpoints(
                other_source_file, [breakpoint_line]
            )
            self.assertEqual(
                len(breakpoint_ids), 1, "expect correct number of breakpoints"
            )

            self.continue_to_breakpoints(breakpoint_ids)

        frames = self.get_stackFrames()
        self.assertLessEqual(2, len(frames), "expect at least 2 frames")

        self.assertIn(
            "path",
            frames[0]["source"],
            "Expect source path to always be in frame (other.c)",
        )
        self.assertIn(
            "path",
            frames[1]["source"],
            "Expect source path in always be in frame (main.c)",
        )

        return frames

    def verify_frames_source(
        self, frames, main_frame_assembly: bool, other_frame_assembly: bool
    ):
        self.assertLessEqual(2, len(frames), "expect at least 2 frames")
        source_0 = frames[0].get("source")
        source_1 = frames[1].get("source")
        self.assertIsNotNone(source_0, "Expects a source object in frame 0")
        self.assertIsNotNone(source_1, "Expects a source object in frame 1")

        # it does not always have a path.
        source_0_path: str = source_0.get("path", "")
        source_1_path: str = source_1.get("path", "")

        if other_frame_assembly:
            self.assertFalse(
                source_0_path.endswith("other.c"),
                "Expect original source path to not be in unavailable source frame (other.c)",
            )
            self.assertIn(
                "sourceReference",
                source_0,
                "Expect sourceReference to be in unavailable source frame (other.c)",
            )
        else:
            self.assertTrue(
                source_0_path.endswith("other.c"),
                "Expect original source path to be in normal source frame (other.c)",
            )
            self.assertNotIn(
                "sourceReference",
                source_0,
                "Expect sourceReference to not be in normal source frame (other.c)",
            )

        if main_frame_assembly:
            self.assertFalse(
                source_1_path.endswith("main.c"),
                "Expect original source path to not be in unavailable source frame (main.c)",
            )
            self.assertIn(
                "sourceReference",
                source_1,
                "Expect sourceReference to be in unavailable source frame (main.c)",
            )
        else:
            self.assertTrue(
                source_1_path.endswith("main.c"),
                "Expect original source path to be in normal source frame (main.c)",
            )
            self.assertNotIn(
                "sourceReference",
                source_1,
                "Expect sourceReference to not be in normal source code frame (main.c)",
            )

    def test_stopDisassemblyDisplay(self):
        """
        Test that with with all stop-disassembly-display values you get correct assembly / no assembly source code.
        """
        self.build_and_run_until_breakpoint()
        frames = self.get_stackFrames()
        self.assertLessEqual(2, len(frames), "expect at least 2 frames")

        self.assertIn(
            "path",
            frames[0]["source"],
            "Expect source path to always be in frame (other.c)",
        )
        self.assertIn(
            "path",
            frames[1]["source"],
            "Expect source path in always be in frame (main.c)",
        )

        self.dap_server.request_evaluate(
            "settings set stop-disassembly-display never", context="repl"
        )
        frames = self.get_stackFrames()
        self.verify_frames_source(
            frames, main_frame_assembly=False, other_frame_assembly=False
        )

        self.dap_server.request_evaluate(
            "settings set stop-disassembly-display always", context="repl"
        )
        frames = self.get_stackFrames()
        self.verify_frames_source(
            frames, main_frame_assembly=True, other_frame_assembly=True
        )

        self.dap_server.request_evaluate(
            "settings set stop-disassembly-display no-source", context="repl"
        )
        frames = self.get_stackFrames()
        self.verify_frames_source(
            frames, main_frame_assembly=False, other_frame_assembly=True
        )

        self.dap_server.request_evaluate(
            "settings set stop-disassembly-display no-debuginfo", context="repl"
        )
        frames = self.get_stackFrames()
        self.verify_frames_source(
            frames, main_frame_assembly=False, other_frame_assembly=False
        )
        self.continue_to_exit()



class myIntSynthProvider(object):
    def __init__(self, valobj, dict):
        self.valobj = valobj
        self.val = self.valobj.GetChildMemberWithName("theValue")

    def num_children(self):
        return 0

    def get_child_at_index(self, index):
        return None

    def get_child_index(self, name):
        return None

    def update(self):
        return False

    def has_children(self):
        return False

    def get_value(self):
        return self.val


class myArraySynthProvider(object):
    def __init__(self, valobj, dict):
        self.valobj = valobj
        self.array = self.valobj.GetChildMemberWithName("array")

    def num_children(self, max_count):
        if 16 < max_count:
            return 16
        return max_count

    def get_child_at_index(self, index):
        return None  # Keep it simple when this is not tested here.

    def get_child_index(self, name):
        return None  # Keep it simple when this is not tested here.

    def has_children(self):
        return True



"""
Test that we stop at runtime instrumentation locations (ubsan).
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbdap_testcase


class TestDAP_ubsan(lldbdap_testcase.DAPTestCaseBase):
    @skipUnlessUndefinedBehaviorSanitizer
    def test_ubsan(self):
        """
        Test that we stop at ubsan.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        self.do_continue()

        self.verify_stop_exception_info("Out of bounds index")
        exceptionInfo = self.get_exceptionInfo()
        self.assertEqual(exceptionInfo["breakMode"], "always")
        self.assertRegex(exceptionInfo["description"], r"Out of bounds index")
        self.assertEqual(exceptionInfo["exceptionId"], "runtime-instrumentation")

        # FIXME: Check on non macOS platform the stop information location heuristic
        # may be wrong. enable when we have updated Ubsan stopInfo heuristic.
        if self.platformIsDarwin():
            self.assertIn("main.c", exceptionInfo["details"]["stackTrace"])



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestRestartBug(GDBRemoteTestBase):
    @expectedFailureAll(bugnumber="llvm.org/pr24530")
    def test(self):
        """
        Test auto-continue behavior when a process is interrupted to deliver
        an "asynchronous" packet. This simulates the situation when a process
        stops on its own just as lldb client is about to interrupt it. The
        client should not auto-continue in this case, unless the user has
        explicitly requested that we ignore signals of this type.
        """

        class MyResponder(MockGDBServerResponder):
            continueCount = 0

            def setBreakpoint(self, packet):
                return "OK"

            def interrupt(self):
                # Simulate process stopping due to a raise(SIGINT) just as lldb
                # is about to interrupt it.
                return "T02reason:signal"

            def cont(self):
                self.continueCount += 1
                if self.continueCount == 1:
                    # No response, wait for the client to interrupt us.
                    return None
                return "W00"  # Exit

        self.server.responder = MyResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)
        self.dbg.SetAsync(True)
        process.Continue()

        # resume the process and immediately try to set another breakpoint. When using the remote
        # stub, this will trigger a request to stop the process.  Make sure we
        # do not lose this signal.
        bkpt = target.BreakpointCreateByAddress(0x1234)
        self.assertTrue(bkpt.IsValid())
        self.assertEqual(bkpt.GetNumLocations(), 1)

        event = lldb.SBEvent()
        while self.dbg.GetListener().WaitForEvent(2, event):
            if self.TraceOn():
                print(
                    "Process changing state to:",
                    self.dbg.StateAsCString(process.GetStateFromEvent(event)),
                )
            if process.GetStateFromEvent(event) == lldb.eStateExited:
                break

        # We should get only one continue packet as the client should not
        # auto-continue after setting the breakpoint.
        self.assertEqual(self.server.responder.continueCount, 1)
        # And the process should end up in the stopped state.
        self.assertState(process.GetState(), lldb.eStateStopped)



"""
Test lldb-dap completions request
"""

# FIXME: remove when LLDB_MINIMUM_PYTHON_VERSION > 3.8
from __future__ import annotations

import json
from typing import Optional
import lldbdap_testcase
from dataclasses import dataclass, replace, asdict
from lldbsuite.test.decorators import skipIf
from lldbsuite.test.lldbtest import line_number


@dataclass(frozen=True)
class CompletionItem:
    label: str
    text: Optional[str] = None
    detail: Optional[str] = None
    start: Optional[int] = None
    length: int = 0

    def __repr__(self):
        # use json as it easier to see the diff on failure.
        return json.dumps(asdict(self), indent=4)

    def clone(self, **kwargs) -> CompletionItem:
        """Creates a copy of this CompletionItem with specified fields modified."""
        return replace(self, **kwargs)


@dataclass(frozen=True)
class Scenario:
    input: str
    expected: set[CompletionItem]
    not_expected: Optional[set[CompletionItem]] = None


session_completion = CompletionItem(
    label="session",
    detail="Commands controlling LLDB session.",
)
settings_completion = CompletionItem(
    label="settings",
    detail="Commands for managing LLDB settings.",
)
memory_completion = CompletionItem(
    label="memory",
    detail="Commands for operating on memory in the current target process.",
)
command_var_completion = CompletionItem(
    label="var",
    detail="Show variables for the current stack frame. Defaults to all arguments and local variables in scope. Names of argument, local, file static and file global variables can be specified.",
    length=3,
)
variable_var_completion = CompletionItem(label="var", detail="vector<baz> &", length=3)
variable_var1_completion = CompletionItem(label="var1", detail="int &")
variable_var2_completion = CompletionItem(label="var2", detail="int &")

str1_completion = CompletionItem(
    label="str1",
    detail="std::string &",
)


# Older version of libcxx produce slightly different typename strings for
# templates like vector.
@skipIf(compiler="clang", compiler_version=["<", "16.0"])
class TestDAP_completions(lldbdap_testcase.DAPTestCaseBase):
    def verify_completions(self, case: Scenario):
        completions = {
            CompletionItem(**comp)
            for comp in self.dap_server.get_completions(case.input)
        }

        # handle expected completions
        for exp_comp in case.expected:
            self.assertIn(
                exp_comp, completions, f"\nCompletion for input: {case.input}"
            )

        # unexpected completions
        for not_exp_comp in case.not_expected or set():
            with self.subTest(f"Not expected completion : {not_exp_comp}"):
                self.assertNotIn(not_exp_comp, completions)

    def setup_debuggee(self):
        program = self.getBuildArtifact("a.out")
        source = "main.cpp"
        self.build_and_launch(program)
        self.set_source_breakpoints(
            source,
            [
                line_number(source, "// breakpoint 1"),
                line_number(source, "// breakpoint 2"),
            ],
        )

    def verify_non_ascii_completion(self, alias_cmd: str):
        """Creates an command alias for the `next` command and
        verify if it has completion for the command and its help.

        It assumes we are in command mode in the repl.
        """
        res = self.dap_server.request_evaluate(
            f"command alias {alias_cmd} next", context="repl"
        )
        self.assertTrue(res["success"])

        part = alias_cmd[:2]  # first two characters
        part_codeunits = len(part.encode("utf-16-le")) // 2

        next_detail = "Source level single step, stepping over calls.  Defaults to current thread unless specified."
        expected_item = CompletionItem(
            label=alias_cmd, detail=next_detail, length=part_codeunits
        )

        # complete the command
        self.verify_completions(Scenario(input=part, expected={expected_item}))
        # complete the help
        self.verify_completions(
            Scenario(input=f"help {part}", expected={expected_item})
        )

        # remove the alias
        res = self.dap_server.request_evaluate(
            f"command unalias {alias_cmd}", context="repl"
        )
        self.assertTrue(res["success"])

    def test_command_completions(self):
        """
        Tests completion requests for lldb commands, within "repl-mode=command"
        """
        self.setup_debuggee()
        self.continue_to_next_stop()

        res = self.dap_server.request_evaluate(
            "`lldb-dap repl-mode command", context="repl"
        )
        self.assertTrue(res["success"])

        # Provides completion for top-level commands
        self.verify_completions(
            Scenario(
                input="se",
                expected={
                    session_completion.clone(length=2),
                    settings_completion.clone(length=2),
                },
            )
        )
        # Provides completions for sub-commands
        self.verify_completions(
            Scenario(
                input="memory ",
                expected={
                    CompletionItem(
                        label="read",
                        detail="Read from the memory of the current target process.",
                    ),
                    CompletionItem(
                        label="region",
                        detail="Get information on the memory region containing an address "
                        "in the current target process.\nIf this command is given an "
                        "<address-expression> once and then repeated without options, "
                        "it will try to print the memory region that follows the "
                        "previously printed region. The command can be repeated "
                        "until the end of the address range is reached.",
                    ),
                },
            ),
        )

        # Provides completions for parameter values of commands
        self.verify_completions(
            Scenario(
                input="`log enable  ", expected={CompletionItem(label="gdb-remote")}
            )
        )

        # Also works if the escape prefix is used
        self.verify_completions(
            Scenario(input="`mem", expected={memory_completion.clone(length=3)})
        )

        self.verify_completions(
            Scenario(
                input="`",
                expected={session_completion, settings_completion, memory_completion},
            )
        )

        # Completes an incomplete quoted token
        self.verify_completions(
            Scenario(
                input='setting "se',
                expected={
                    CompletionItem(
                        label="set",
                        detail="Set the value of the specified debugger setting.",
                        length=3,
                    )
                },
            )
        )

        # Completes an incomplete quoted token
        self.verify_completions(
            Scenario(input="'mem", expected={memory_completion.clone(length=4)})
        )

        # Completes expressions with quotes inside
        self.verify_completions(
            Scenario(
                input='expr " "; typed',
                expected={CompletionItem(label="typedef", length=5)},
            )
        )

        # Provides completions for commands, but not variables
        self.verify_completions(
            Scenario(
                input="var",
                expected={command_var_completion},
                not_expected={variable_var_completion},
            )
        )

        # Completes partial completion
        self.verify_completions(
            Scenario(
                input="plugin list ar",
                expected={CompletionItem(label="architecture", length=2)},
            )
        )

        # Complete custom command with non ascii character.
        self.verify_non_ascii_completion("n€xt")  # 2 bytes £
        self.verify_non_ascii_completion("n£xt")  # 3 bytes €
        self.verify_non_ascii_completion("n💩xt")  # 4 bytes 💩
        self.verify_non_ascii_completion("√∂xt")  # start with non ascii
        self.verify_non_ascii_completion("one_seç")  # ends with non ascii

    def test_variable_completions(self):
        """
        Tests completion requests in "repl-mode=variable"
        """
        self.setup_debuggee()
        self.continue_to_next_stop()

        res = self.dap_server.request_evaluate(
            "`lldb-dap repl-mode variable", context="repl"
        )
        self.assertTrue(res["success"])

        # Provides completions for varibles, but not command
        self.verify_completions(
            Scenario(
                input="var",
                expected={variable_var_completion},
                not_expected={command_var_completion},
            )
        )

        # We stopped inside `fun`, so we shouldn't see variables from main
        self.verify_completions(
            Scenario(
                input="var",
                expected={variable_var_completion},
                not_expected={
                    variable_var1_completion.clone(length=3),
                    variable_var2_completion.clone(length=3),
                },
            )
        )

        # We should see global keywords but not variables inside main
        self.verify_completions(
            Scenario(
                input="str",
                expected={CompletionItem(label="struct", length=3)},
                not_expected={str1_completion.clone(length=3)},
            )
        )

        self.continue_to_next_stop()

        # We stopped in `main`, so we should see variables from main but
        # not from the other function
        self.verify_completions(
            Scenario(
                input="var",
                expected={
                    variable_var1_completion.clone(length=3),
                    variable_var2_completion.clone(length=3),
                },
                not_expected={
                    variable_var_completion.clone(length=3),
                },
            )
        )

        self.verify_completions(
            Scenario(
                input="str",
                expected={
                    CompletionItem(label="struct", length=3),
                    str1_completion.clone(length=3),
                },
            )
        )

        self.assertIsNotNone(self.dap_server.get_completions("ƒ"))
        # Test utf8 after ascii.
        # TODO
        self.dap_server.get_completions("mƒ")

        # Completion also works for more complex expressions
        self.verify_completions(
            Scenario(
                input="foo1.v",
                expected={CompletionItem(label="foo1.var1", detail="int", length=6)},
            )
        )

        self.verify_completions(
            Scenario(
                input="foo1.my_bar_object.v",
                expected={
                    CompletionItem(
                        label="foo1.my_bar_object.var1", detail="int", length=20
                    )
                },
            )
        )

        self.verify_completions(
            Scenario(
                input="foo1.var1 + foo1.v",
                expected={CompletionItem(label="foo1.var1", detail="int", length=6)},
            )
        )

        self.verify_completions(
            Scenario(
                input="foo1.var1 + v",
                expected={CompletionItem(label="var1", detail="int &", length=1)},
            )
        )

        # should correctly handle spaces between objects and member operators
        self.verify_completions(
            Scenario(
                input="foo1 .v",
                expected={CompletionItem(label=".var1", detail="int", length=2)},
                not_expected={CompletionItem(label=".var2", detail="int", length=2)},
            )
        )

        self.verify_completions(
            Scenario(
                input="foo1 . v",
                expected={CompletionItem(label="var1", detail="int", length=1)},
                not_expected={CompletionItem(label="var2", detail="int", length=1)},
            )
        )

        # Even in variable mode, we can still use the escape prefix
        self.verify_completions(
            Scenario(input="`mem", expected={memory_completion.clone(length=3)})
        )

    def test_auto_completions(self):
        """
        Tests completion requests in "repl-mode=auto"
        """
        self.setup_debuggee()

        res = self.dap_server.request_evaluate(
            "`lldb-dap repl-mode auto", context="repl"
        )
        self.assertTrue(res["success"])

        self.continue_to_next_stop()

        # Stopped at breakpoint 1
        # 'var' variable is in scope, completions should not show any warning.
        self.dap_server.get_completions("var ")
        self.continue_to_next_stop()

        # We are stopped inside `main`. Variables `var1` and `var2` are in scope.
        # Make sure, we offer all completions
        self.verify_completions(
            Scenario(
                input="va",
                expected={
                    command_var_completion.clone(length=2),
                    variable_var1_completion.clone(length=2),
                    variable_var2_completion.clone(length=2),
                },
            )
        )

        # If we are using the escape prefix, only commands are suggested, but no variables
        self.verify_completions(
            Scenario(
                input="`va",
                expected={
                    command_var_completion.clone(length=2),
                },
                not_expected={
                    variable_var1_completion.clone(length=2),
                    variable_var2_completion.clone(length=2),
                },
            )
        )

        # TODO: Note we are not checking the result because the `expression --` command adds an extra character
        # for non ascii variables.
        self.assertIsNotNone(self.dap_server.get_completions("ƒ"))

        self.continue_to_exit()
        console_str = self.get_console()
        # we check in console to avoid waiting for output event.
        self.assertNotIn(
            "Expression 'var' is both an LLDB command and variable", console_str
        )



"""
Test lldb Python API SBValue::Cast(SBType) for C++ types.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CppValueCastTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipIf(bugnumber="llvm.org/PR36714")
    @add_test_categories(["pyapi"])
    def test_value_cast_with_virtual_inheritance(self):
        """Test SBValue::Cast(SBType) API for C++ types with virtual inheritance."""
        self.build(dictionary=self.d_virtual)
        self.setTearDownCleanup(dictionary=self.d_virtual)
        self.do_sbvalue_cast(self.exe_name)

    @add_test_categories(["pyapi"])
    def test_value_cast_with_regular_inheritance(self):
        """Test SBValue::Cast(SBType) API for C++ types with regular inheritance."""
        self.build(dictionary=self.d_regular)
        self.setTearDownCleanup(dictionary=self.d_regular)
        self.do_sbvalue_cast(self.exe_name)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Find the line number to break for main.c.
        self.source = "sbvalue-cast.cpp"
        self.line = line_number(self.source, "// Set breakpoint here.")
        self.exe_name = self.testMethodName
        self.d_virtual = {
            "CXX_SOURCES": self.source,
            "EXE": self.exe_name,
            "CFLAGS_EXTRAS": "-DDO_VIRTUAL_INHERITANCE",
        }
        self.d_regular = {"CXX_SOURCES": self.source, "EXE": self.exe_name}

    def do_sbvalue_cast(self, exe_name):
        """Test SBValue::Cast(SBType) API for C++ types."""
        exe = self.getBuildArtifact(exe_name)

        # Create a target from the debugger.

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set up our breakpoints:

        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        # Find DerivedA and DerivedB types.
        typeA = target.FindFirstType("DerivedA")
        typeB = target.FindFirstType("DerivedB")
        self.DebugSBType(typeA)
        self.DebugSBType(typeB)
        self.assertTrue(typeA)
        self.assertTrue(typeB)
        error = lldb.SBError()

        # First stop is for DerivedA instance.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint)
        self.assertEqual(len(threads), 1)
        thread = threads[0]
        frame0 = thread.GetFrameAtIndex(0)

        tellerA = frame0.FindVariable("teller", lldb.eNoDynamicValues)
        self.DebugSBValue(tellerA)
        self.assertEqual(
            tellerA.GetChildMemberWithName("m_base_val").GetValueAsUnsigned(error, 0),
            20,
        )

        if self.TraceOn():
            for child in tellerA:
                print("child name:", child.GetName())
                print(child)

        # Call SBValue.Cast() to obtain instanceA.
        instanceA = tellerA.Cast(typeA.GetPointerType())
        self.DebugSBValue(instanceA)

        # Iterate through all the children and print their values.
        if self.TraceOn():
            for child in instanceA:
                print("child name:", child.GetName())
                print(child)
        a_member_val = instanceA.GetChildMemberWithName("m_a_val")
        self.DebugSBValue(a_member_val)
        self.assertEqual(a_member_val.GetValueAsUnsigned(error, 0), 10)

        # Second stop is for DerivedB instance.
        threads = lldbutil.continue_to_breakpoint(process, breakpoint)
        self.assertEqual(len(threads), 1)
        thread = threads[0]
        frame0 = thread.GetFrameAtIndex(0)

        tellerB = frame0.FindVariable("teller", lldb.eNoDynamicValues)
        self.DebugSBValue(tellerB)
        self.assertEqual(
            tellerB.GetChildMemberWithName("m_base_val").GetValueAsUnsigned(error, 0),
            12,
        )

        if self.TraceOn():
            for child in tellerB:
                print("child name:", child.GetName())
                print(child)

        # Call SBValue.Cast() to obtain instanceB.
        instanceB = tellerB.Cast(typeB.GetPointerType())
        self.DebugSBValue(instanceB)

        # Iterate through all the children and print their values.
        if self.TraceOn():
            for child in instanceB:
                print("child name:", child.GetName())
                print(child)
        b_member_val = instanceB.GetChildMemberWithName("m_b_val")
        self.DebugSBValue(b_member_val)
        self.assertEqual(b_member_val.GetValueAsUnsigned(error, 0), 36)



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetID()
    obj.IsValid()
    obj.GetWatchAddress()
    obj.GetWatchSize()
    obj.SetEnabled(True)
    obj.IsEnabled()
    obj.GetHitCount()
    obj.GetIgnoreCount()
    obj.SetIgnoreCount(5)
    obj.GetDescription(lldb.SBStream(), lldb.eDescriptionLevelVerbose)
    obj.SetCondition("shouldWeStop()")
    obj.GetCondition()



#!/usr/bin/env python3
"""
Usage: -i <path/to/input-header.h> -o <path/to/output-header.h> -m LLDB_MAJOR_VERSION -n LLDB_MINOR_VERSION -p LLDB_PATCH_VERSION

This script uncomments and populates the versioning information in lldb-defines.h. Note that the LLDB version numbering looks like MAJOR.MINOR.PATCH
"""

import argparse
import os
import re

LLDB_VERSION_REGEX = re.compile(r"//\s*#define LLDB_VERSION\s*$", re.M)
LLDB_REVISION_REGEX = re.compile(r"//\s*#define LLDB_REVISION\s*$", re.M)
LLDB_VERSION_STRING_REGEX = re.compile(r"//\s*#define LLDB_VERSION_STRING\s*$", re.M)


def main():
    parser = argparse.ArgumentParser(
        description="This script uncomments and populates the versioning information in lldb-defines.h. Note that the LLDB version numbering looks like MAJOR.MINOR.PATCH"
    )
    parser.add_argument("-i", "--input_path", help="The filepath for the input header.")
    parser.add_argument(
        "-o", "--output_path", help="The filepath for the output header."
    )
    parser.add_argument("-m", "--major", help="The LLDB version major.")
    parser.add_argument("-n", "--minor", help="The LLDB version minor.")
    parser.add_argument("-p", "--patch", help="The LLDB version patch number.")
    args = parser.parse_args()
    input_path = str(args.input_path)
    output_path = str(args.output_path)

    # Create the output dir if it doesn't already exist
    if not os.path.exists(os.path.dirname(output_path)):
        os.makedirs(os.path.dirname(output_path))

    with open(input_path, "r") as input_file:
        lines = input_file.readlines()
        file_buffer = "".join(lines)

    with open(output_path, "w") as output_file:
        # For the defines in lldb-defines.h that define the major, minor and version string
        # uncomment each define and populate its value using the arguments passed in.
        # e.g. //#define LLDB_VERSION -> #define LLDB_VERSION <LLDB_MAJOR_VERSION>
        file_buffer = re.sub(
            LLDB_VERSION_REGEX,
            r"#define LLDB_VERSION " + args.major,
            file_buffer,
        )

        file_buffer = re.sub(
            LLDB_REVISION_REGEX,
            r"#define LLDB_REVISION " + args.patch,
            file_buffer,
        )
        file_buffer = re.sub(
            LLDB_VERSION_STRING_REGEX,
            r'#define LLDB_VERSION_STRING "{0}.{1}.{2}"'.format(
                args.major, args.minor, args.patch
            ),
            file_buffer,
        )
        output_file.write(file_buffer)


if __name__ == "__main__":
    main()



# encoding: utf-8
"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCDataFormatterTestCase import ObjCDataFormatterTestCase


class ObjCDataFormatterNSURL(ObjCDataFormatterTestCase):
    SHARED_BUILD_TESTCASE = False

    def test_nsurl_with_run_command(self):
        """Test formatters for NSURL."""
        self.appkit_tester_impl(self.nsurl_data_formatter_commands, True)

    @skipUnlessDarwin
    def test_nsurl_with_run_command_no_const(self):
        """Test formatters for NSURL."""
        self.appkit_tester_impl(self.nsurl_data_formatter_commands, False)

    def nsurl_data_formatter_commands(self):
        self.expect(
            "frame variable cfurl_ref cfchildurl_ref cfgchildurl_ref",
            substrs=[
                "(CFURLRef) cfurl_ref = ",
                '@"http://www.foo.bar',
                "cfchildurl_ref = ",
                '@"page.html -- http://www.foo.bar',
                "(CFURLRef) cfgchildurl_ref = ",
                '@"?whatever -- http://www.foo.bar/page.html"',
            ],
        )

        self.expect(
            "frame variable nsurl nsurl2 nsurl3",
            substrs=[
                "(NSURL *) nsurl = ",
                '@"http://www.foo.bar',
                "(NSURL *) nsurl2 =",
                '@"page.html -- http://www.foo.bar',
                "(NSURL *) nsurl3 = ",
                '@"?whatever -- http://www.foo.bar/page.html"',
            ],
        )



"""
Test that lldb can continue past a __builtin_debugtrap, but not a __builtin_trap
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class BuiltinDebugTrapTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # Currently this depends on behavior in debugserver to
    # advance the pc past __builtin_trap instructions so that
    # continue works.  Everyone is in agreement that this
    # should be moved up into lldb instead of depending on the
    # remote stub rewriting the pc values.
    @skipUnlessDarwin
    def test(self):
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        # Continue to __builtin_debugtrap()
        process.Continue()
        if self.TraceOn():
            self.runCmd("f")
            self.runCmd("bt")
            self.runCmd("ta v global")

        self.assertEqual(
            process.GetSelectedThread().GetStopReason(), lldb.eStopReasonException
        )

        list = target.FindGlobalVariables("global", 1, lldb.eMatchTypeNormal)
        self.assertEqual(list.GetSize(), 1)
        global_value = list.GetValueAtIndex(0)

        self.assertEqual(global_value.GetValueAsUnsigned(), 5)

        # Continue to the __builtin_trap() -- we should be able to
        # continue past __builtin_debugtrap.
        process.Continue()
        if self.TraceOn():
            self.runCmd("f")
            self.runCmd("bt")
            self.runCmd("ta v global")

        self.assertEqual(
            process.GetSelectedThread().GetStopReason(), lldb.eStopReasonException
        )

        # "global" is now 10.
        self.assertEqual(global_value.GetValueAsUnsigned(), 10)

        # We should be at the same point as before -- cannot advance
        # past a __builtin_trap().
        process.Continue()
        if self.TraceOn():
            self.runCmd("f")
            self.runCmd("bt")
            self.runCmd("ta v global")

        self.assertEqual(
            process.GetSelectedThread().GetStopReason(), lldb.eStopReasonException
        )

        # "global" is still 10.
        self.assertEqual(global_value.GetValueAsUnsigned(), 10)



"""
Test controlling `expression` result variables are persistent.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.c"))

    def test_enable_persistent_result(self):
        """Test explicitly enabling result variables persistence."""
        self.expect("expression --persistent-result on -- i", substrs=["(int) $0 = 30"])
        # Verify the lifetime of $0 extends beyond the `expression` it was created in.
        self.expect("expression $0", substrs=["(int) $0 = 30"])

    def test_disable_persistent_result(self):
        """Test explicitly disabling persistent result variables."""
        self.expect("expression --persistent-result off -- i", substrs=["(int) 30"])
        # Verify a persistent result was not silently created.
        self.expect("expression $0", error=True)

    def test_expression_persists_result(self):
        """Test `expression`'s default behavior is to persist a result variable."""
        self.expect("expression i", substrs=["(int) $0 = 30"])
        self.expect("expression $0", substrs=["(int) $0 = 30"])

    def test_p_does_not_persist_results(self):
        """Test `p` does not persist a result variable."""
        self.expect("p i", startstr="(int) 30")
        self.expect("p $0", error=True)



import re
import os
import subprocess

from .builder import Builder
from lldbsuite.test import configuration
import lldbsuite.test.lldbutil as lldbutil

REMOTE_PLATFORM_NAME_RE = re.compile(r"^remote-(.+)$")
SIMULATOR_PLATFORM_RE = re.compile(r"^(.+)-simulator$")


def get_os_env_from_platform(platform):
    match = REMOTE_PLATFORM_NAME_RE.match(platform)
    if match:
        return match.group(1), ""
    match = SIMULATOR_PLATFORM_RE.match(platform)
    if match:
        return match.group(1), "simulator"
    return None, None


def get_os_from_sdk(sdk):
    return sdk[: sdk.find(".")], ""


def get_os_and_env():
    if configuration.lldb_platform_name:
        return get_os_env_from_platform(configuration.lldb_platform_name)
    if configuration.apple_sdk:
        return get_os_from_sdk(configuration.apple_sdk)
    return None, None


def get_triple():
    # Construct the vendor component.
    vendor = "apple"

    # Construct the os component.
    os, env = get_os_and_env()
    if os is None or env is None:
        return None, None, None, None

    # Get the SDK from the os and env.
    sdk = lldbutil.get_xcode_sdk(os, env)
    if sdk is None:
        return None, None, None, None

    # Get the version from the SDK.
    version = lldbutil.get_xcode_sdk_version(sdk)
    if version is None:
        return None, None, None, None

    return vendor, os, version, env


def get_triple_str(arch, vendor, os, version, env):
    if None in [arch, vendor, os, version, env]:
        return None

    component = [arch, vendor, os + version]
    if env:
        component.append(env)
    return "-".join(component)


class BuilderDarwin(Builder):
    def getTriple(self, arch):
        vendor, os, version, env = get_triple()
        return get_triple_str(arch, vendor, os, version, env)

    def getExtraMakeArgs(self):
        """
        Helper function to return extra argumentsfor the make system. This
        method is meant to be overridden by platform specific builders.
        """
        args = dict()

        if configuration.dsymutil:
            args["DSYMUTIL"] = configuration.dsymutil

        if configuration.apple_sdk and "internal" in configuration.apple_sdk:
            sdk_root = lldbutil.get_xcode_sdk_root(configuration.apple_sdk)
            if sdk_root:
                private_frameworks = os.path.join(
                    sdk_root, "System", "Library", "PrivateFrameworks"
                )
                args["FRAMEWORK_INCLUDES"] = "-F{}".format(private_frameworks)

        operating_system, env = get_os_and_env()

        builder_dir = os.path.dirname(os.path.abspath(__file__))
        test_dir = os.path.dirname(builder_dir)
        if not operating_system:
            entitlements_file = "entitlements-macos.plist"
        else:
            if env == "simulator":
                entitlements_file = "entitlements-simulator.plist"
            else:
                entitlements_file = "entitlements.plist"
        entitlements = os.path.join(test_dir, "make", entitlements_file)
        args["CODESIGN"] = "codesign --entitlements {}".format(entitlements)

        # Return extra args as a formatted string.
        return ["{}={}".format(key, value) for key, value in args.items()]

    def getArchCFlags(self, arch):
        """Returns the ARCH_CFLAGS for the make system."""
        # Get the triple components.
        vendor, os, version, env = get_triple()
        triple = get_triple_str(arch, vendor, os, version, env)
        if not triple:
            return []

        # Construct min version argument
        version_min = ""
        if env == "simulator":
            version_min = "-m{}-simulator-version-min={}".format(os, version)
        else:
            version_min = "-m{}-version-min={}".format(os, version)

        return ["ARCH_CFLAGS=-target {} {}".format(triple, version_min)]

    def _getDebugInfoArgs(self, debug_info):
        if debug_info == "dsym":
            return ["MAKE_DSYM=YES"]
        return super(BuilderDarwin, self)._getDebugInfoArgs(debug_info)



"""
Test the ptr_refs tool on Darwin
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPtrRefs(TestBase):
    @skipIfAsan  # The output looks different under ASAN.
    @skipIfMTE  # Heap scanning reads tagged memory with untagged pointers.
    @skipUnlessDarwin
    def test_ptr_refs(self):
        """Test format string functionality."""
        self.build()

        lldbutil.run_to_source_breakpoint(self, "break", lldb.SBFileSpec("main.c"))

        self.runCmd("command script import lldb.macosx.heap")
        self.expect("ptr_refs my_ptr", substrs=["malloc", "stack"])



"""Check that we handle an ImportError in a special way when command script importing files."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class Rdar12586188TestCase(TestBase):
    @add_test_categories(["pyapi"])
    @no_debug_info_test
    def test_rdar12586188_command(self):
        """Check that we handle an ImportError in a special way when command script importing files."""
        self.run_test()

    def run_test(self):
        """Check that we handle an ImportError in a special way when command script importing files."""

        self.expect(
            "command script import ./fail12586188.py --allow-reload",
            error=True,
            substrs=['raise ImportError("I do not want to be imported")'],
        )
        self.expect(
            "command script import ./fail212586188.py --allow-reload",
            error=True,
            substrs=['raise ValueError("I do not want to be imported")'],
        )



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DataFormatterOneIsSingularTestCase(TestBase):
    @skipUnlessDarwin
    def test_one_is_singular_with_run_command(self):
        """Test that 1 item is not as reported as 1 items."""
        self.build()
        self.oneness_data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.m", "// Set break point at this line.")

    def oneness_data_formatter_commands(self):
        """Test that 1 item is not as reported as 1 items."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Now check that we are displaying Cocoa classes correctly
        self.expect("frame variable key", substrs=['@"1 element"'])
        self.expect("frame variable key", matching=False, substrs=["1 elements"])
        self.expect("frame variable value", substrs=['@"1 element"'])
        self.expect("frame variable value", matching=False, substrs=["1 elements"])
        self.expect("frame variable dict", substrs=["1 key/value pair"])
        self.expect(
            "frame variable dict", matching=False, substrs=["1 key/value pairs"]
        )
        self.expect("frame variable imset", substrs=["1 index"])
        self.expect("frame variable imset", matching=False, substrs=["1 indexes"])
        self.expect("frame variable binheap_ref", substrs=['@"1 item"'])
        self.expect("frame variable binheap_ref", matching=False, substrs=["1 items"])
        self.expect("frame variable immutableData", substrs=["1 byte"])
        self.expect("frame variable immutableData", matching=False, substrs=["1 bytes"])



import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestCase(TestBase):
    def test_raw_bytes(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.c"))
        self.runCmd("type summary add -s '${var.ubyte:x-2}${var.sbyte:x-2}!' Bytes")
        self.expect("v bytes", substrs=[" = 3001!"])

    def test_bad_format(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.c"))
        self.expect(
            "type summary add -s '${var.ubyte:y}!' Bytes",
            error=True,
            substrs=["invalid llvm format"],
        )



"""
Test floating point expressions with zero, NaN, dernormalized and infinite
numbers.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FPNaNTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break inside the main.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1
        )

        self.runCmd("run", RUN_SUCCEEDED)
        # Zero and denorm
        self.expect(
            "expr +0.0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["double", "0"],
        )
        self.expect(
            "expr -0.0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["double", "0"],
        )
        self.expect(
            "expr 0.0 / 0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["double", "NaN"],
        )
        self.expect(
            "expr 0 / 0.0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["double", "NaN"],
        )
        self.expect(
            "expr 1 / +0.0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["double", "+Inf"],
        )
        self.expect(
            "expr 1 / -0.0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["double", "-Inf"],
        )
        self.expect(
            "expr +0.0 / +0.0 != +0.0 / +0.0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "true"],
        )
        self.expect(
            "expr -1.f * 0",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["float", "-0"],
        )
        self.expect(
            "expr 0x0.123p-1",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["double", "0.0355224609375"],
        )
        # NaN
        self.expect(
            "expr fnan < fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "false"],
        )
        self.expect(
            "expr fnan <= fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "false"],
        )
        self.expect(
            "expr fnan > fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "false"],
        )
        self.expect(
            "expr fnan >= fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "false"],
        )
        self.expect(
            "expr fnan == fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "false"],
        )
        self.expect(
            "expr fnan != fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "true"],
        )
        self.expect(
            "expr 1.0 <= fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "false"],
        )
        self.expect(
            "expr 1.0f < fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "false"],
        )
        self.expect(
            "expr 1.0f != fnan",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["bool", "true"],
        )
        self.expect(
            "expr (unsigned int) fdenorm",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["int", "0"],
        )
        self.expect(
            "expr (unsigned int) (1.0f + fdenorm)",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["int", "1"],
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestTaggedPointerCommand(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.m")
        )

        n1 = thread.GetSelectedFrame().FindVariable("n1")
        self.expect(
            f"lang objc tagged-pointer info {n1.addr}",
            substrs=[
                f"{n1.addr} is tagged",
                "payload = 0x0000000000000012",
                "value = 0x0000000000000001",
                "info bits = 0x0000000000000002",
                "class = __NSCFNumber",
            ],
        )

        self.expect(
            "lang objc tagged-pointer info bogus",
            error=True,
            patterns=["could not convert 'bogus' to a valid address"],
        )

        self.expect(
            "lang objc tagged-pointer info 0x0",
            error=True,
            patterns=["could not convert '0x0' to a valid address"],
        )



"""
Tests loading of classes when the loading is triggered via a typedef inside the
class (and not via the normal LLDB lookup that first resolves the surrounding
class).
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        self.createTestTarget()

        # Print the top-level typedef which triggers the loading of the class
        # that the typedef is defined inside.
        self.expect_expr(
            "pull_in_classes",
            result_type="StructWithMember::MemberTypedef",
            result_value="0",
        )

        # Print the classes and check their types.
        self.expect_expr(
            "struct_to_print",
            result_type="StructWithMember",
            result_children=[
                ValueCheck(
                    name="m",
                    type="StructWithNested::Nested<int>::OtherTypedef",
                    children=[ValueCheck(name="i", value="0", type="int")],
                )
            ],
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ValueAPIEmptyClassTestCase(TestBase):
    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        line = line_number("main.cpp", "// Break at this line")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Create the breakpoint inside function 'main'.
        breakpoint = target.BreakpointCreateByLocation("main.cpp", line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Get Frame #0.
        self.assertState(process.GetState(), lldb.eStateStopped)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(),
            "There should be a thread stopped due to breakpoint condition",
        )
        frame0 = thread.GetFrameAtIndex(0)

        # Verify that we can access to a frame variable with an empty class type
        e = frame0.FindVariable("e")
        self.assertTrue(e.IsValid(), VALID_VARIABLE)
        self.DebugSBValue(e)
        self.assertEqual(e.GetNumChildren(), 0)

        # Verify that we can acces to a frame variable what is a pointer to an
        # empty class
        ep = frame0.FindVariable("ep")
        self.assertTrue(ep.IsValid(), VALID_VARIABLE)
        self.DebugSBValue(ep)

        # Verify that we can dereference a pointer to an empty class
        epd = ep.Dereference()
        self.assertTrue(epd.IsValid(), VALID_VARIABLE)
        self.DebugSBValue(epd)
        self.assertEqual(epd.GetNumChildren(), 0)



#!/usr/bin/env python

# ----------------------------------------------------------------------
# Be sure to add the python path that points to the LLDB shared library.
#
# # To use this in the embedded python interpreter using "lldb" just
# import it with the full path using the "command script import"
# command
#   (lldb) command script import /path/to/cmdtemplate.py
# ----------------------------------------------------------------------

import platform
import os
import re
import sys
import subprocess

try:
    # Just try for LLDB in case PYTHONPATH is already correctly setup
    import lldb
except ImportError:
    lldb_python_dirs = list()
    # lldb is not in the PYTHONPATH, try some defaults for the current platform
    platform_system = platform.system()
    if platform_system == "Darwin":
        # On Darwin, try the currently selected Xcode directory
        xcode_dir = subprocess.check_output("xcode-select --print-path", shell=True)
        if xcode_dir:
            lldb_python_dirs.append(
                os.path.realpath(
                    xcode_dir + "/../SharedFrameworks/LLDB.framework/Resources/Python"
                )
            )
            lldb_python_dirs.append(
                xcode_dir + "/Library/PrivateFrameworks/LLDB.framework/Resources/Python"
            )
        lldb_python_dirs.append(
            "/System/Library/PrivateFrameworks/LLDB.framework/Resources/Python"
        )
    success = False
    for lldb_python_dir in lldb_python_dirs:
        if os.path.exists(lldb_python_dir):
            if not (sys.path.__contains__(lldb_python_dir)):
                sys.path.append(lldb_python_dir)
                try:
                    import lldb
                except ImportError:
                    pass
                else:
                    print('imported lldb from: "%s"' % (lldb_python_dir))
                    success = True
                    break
    if not success:
        print(
            "error: couldn't locate the 'lldb' module, please set PYTHONPATH correctly"
        )
        sys.exit(1)

import optparse
import shlex
import string
import struct
import time


def append_data_callback(option, opt_str, value, parser):
    if opt_str == "--uint8":
        int8 = int(value, 0)
        parser.values.data += struct.pack("1B", int8)
    if opt_str == "--uint16":
        int16 = int(value, 0)
        parser.values.data += struct.pack("1H", int16)
    if opt_str == "--uint32":
        int32 = int(value, 0)
        parser.values.data += struct.pack("1I", int32)
    if opt_str == "--uint64":
        int64 = int(value, 0)
        parser.values.data += struct.pack("1Q", int64)
    if opt_str == "--int8":
        int8 = int(value, 0)
        parser.values.data += struct.pack("1b", int8)
    if opt_str == "--int16":
        int16 = int(value, 0)
        parser.values.data += struct.pack("1h", int16)
    if opt_str == "--int32":
        int32 = int(value, 0)
        parser.values.data += struct.pack("1i", int32)
    if opt_str == "--int64":
        int64 = int(value, 0)
        parser.values.data += struct.pack("1q", int64)


def create_memfind_options():
    usage = "usage: %prog [options] STARTADDR [ENDADDR]"
    description = """This command can find data in a specified address range.
Options are used to specify the data that is to be looked for and the options
can be specified multiple times to look for longer streams of data.
"""
    parser = optparse.OptionParser(description=description, prog="memfind", usage=usage)
    parser.add_option(
        "-s",
        "--size",
        type="int",
        metavar="BYTESIZE",
        dest="size",
        help="Specify the byte size to search.",
        default=0,
    )
    parser.add_option(
        "--int8",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 8 bit signed integer value to search for in memory.",
        default="",
    )
    parser.add_option(
        "--int16",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 16 bit signed integer value to search for in memory.",
        default="",
    )
    parser.add_option(
        "--int32",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 32 bit signed integer value to search for in memory.",
        default="",
    )
    parser.add_option(
        "--int64",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 64 bit signed integer value to search for in memory.",
        default="",
    )
    parser.add_option(
        "--uint8",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 8 bit unsigned integer value to search for in memory.",
        default="",
    )
    parser.add_option(
        "--uint16",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 16 bit unsigned integer value to search for in memory.",
        default="",
    )
    parser.add_option(
        "--uint32",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 32 bit unsigned integer value to search for in memory.",
        default="",
    )
    parser.add_option(
        "--uint64",
        action="callback",
        callback=append_data_callback,
        type="string",
        metavar="INT",
        dest="data",
        help="Specify a 64 bit unsigned integer value to search for in memory.",
        default="",
    )
    return parser


def memfind_command(debugger, command, result, dict):
    # Use the Shell Lexer to properly parse up command options just like a
    # shell would
    command_args = shlex.split(command)
    parser = create_memfind_options()
    (options, args) = parser.parse_args(command_args)
    # try:
    #     (options, args) = parser.parse_args(command_args)
    # except:
    #     # if you don't handle exceptions, passing an incorrect argument to the OptionParser will cause LLDB to exit
    #     # (courtesy of OptParse dealing with argument errors by throwing SystemExit)
    #     result.SetStatus (lldb.eReturnStatusFailed)
    #     print >>result, "error: option parsing failed" # returning a string is the same as returning an error whose description is the string
    #     return
    memfind(debugger.GetSelectedTarget(), options, args, result)


def print_error(str, show_usage, result):
    print(str, file=result)
    if show_usage:
        print(create_memfind_options().format_help(), file=result)


def memfind(target, options, args, result):
    num_args = len(args)
    start_addr = 0
    if num_args == 1:
        if options.size > 0:
            print_error(
                "error: --size must be specified if there is no ENDADDR argument",
                True,
                result,
            )
            return
        start_addr = int(args[0], 0)
    elif num_args == 2:
        if options.size != 0:
            print_error(
                "error: --size can't be specified with an ENDADDR argument",
                True,
                result,
            )
            return
        start_addr = int(args[0], 0)
        end_addr = int(args[1], 0)
        if start_addr >= end_addr:
            print_error(
                "error: inavlid memory range [%#x - %#x)" % (start_addr, end_addr),
                True,
                result,
            )
            return
        options.size = end_addr - start_addr
    else:
        print_error("error: memfind takes 1 or 2 arguments", True, result)
        return

    if not options.data:
        print("error: no data specified to search for", file=result)
        return

    if not target:
        print("error: invalid target", file=result)
        return
    process = target.process
    if not process:
        print("error: invalid process", file=result)
        return

    error = lldb.SBError()
    bytes = process.ReadMemory(start_addr, options.size, error)
    if error.Success():
        num_matches = 0
        print(
            "Searching memory range [%#x - %#x) for" % (start_addr, end_addr),
            end=" ",
            file=result,
        )
        for byte in options.data:
            print("%2.2x" % ord(byte), end=" ", file=result)
        print(file=result)

        match_index = string.find(bytes, options.data)
        while match_index != -1:
            num_matches = num_matches + 1
            print(
                "%#x: %#x + %u" % (start_addr + match_index, start_addr, match_index),
                file=result,
            )
            match_index = string.find(bytes, options.data, match_index + 1)

        if num_matches == 0:
            print("error: no matches found", file=result)
    else:
        print("error: %s" % (error.GetCString()), file=result)


if __name__ == "__main__":
    print(
        "error: this script is designed to be used within the embedded script interpreter in LLDB"
    )


def __lldb_init_module(debugger, internal_dict):
    memfind_command.__doc__ = create_memfind_options().format_help()
    debugger.HandleCommand("command script add -o -f memory.memfind_command memfind")
    print('"memfind" command installed, use the "--help" option for detailed help')



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GenericOptionalDataFormatterTestCase(TestBase):
    TEST_WITH_PDB_DEBUG_INFO = True
    SHARED_BUILD_TESTCASE = False

    def do_test_with_run_command(self):
        """Test that that file and class static variables display correctly."""

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)

        self.addTearDownHook(cleanup)

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        bkpt = self.target().FindBreakpointByID(
            lldbutil.run_break_set_by_source_regexp(self, "break here")
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.runCmd("frame variable has_optional")

        output = self.res.GetOutput()

        ## The variable has_optional tells us if the test program
        ## detected we have a sufficient libc++ version to support optional
        ## false means we do not and therefore should skip the test
        if output.find("(bool) has_optional = false") != -1:
            self.skipTest("Optional not supported")

        lldbutil.continue_to_breakpoint(self.process(), bkpt)

        self.expect("frame variable number_not_engaged", substrs=["Has Value=false"])

        self.expect(
            "frame variable number_engaged",
            substrs=["Has Value=true", "Value = 42", "}"],
        )

        self.expect(
            "frame var numbers",
            substrs=[
                (
                    "(std::optional<std::vector<int, std::allocator<int>>>) numbers =  Has Value=true  {"
                    if self.getDebugInfo() == "pdb"
                    else "(optional_int_vect) numbers =  Has Value=true  {"
                ),
                "Value = size=4 {",
                "[0] = 1",
                "[1] = 2",
                "[2] = 3",
                "[3] = 4",
                "}",
                "}",
            ],
        )

        self.expect(
            "frame var ostring",
            substrs=[
                (
                    "(std::optional<std::basic_string<char, std::char_traits<char>, std::allocator<char>>>) ostring =  Has Value=true  {"
                    if self.getDebugInfo() == "pdb"
                    else "(optional_string) ostring =  Has Value=true  {"
                ),
                'Value = "hello"',
                "}",
            ],
        )

        self.expect_var_path("*number_engaged", value="42")
        self.expect_var_path("*x", children=[ValueCheck(name="x", value="42")])
        self.expect_var_path("x->x", value="42")

        # The error message could use some improvement, but at least we can
        # check we don't crash.
        self.expect(
            "frame variable *number_not_engaged",
            error=True,
            substrs=["dereference failed: not a pointer, reference or array type"],
        )

    @add_test_categories(["libc++"])
    def test_with_run_command_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["libstdcxx"])
    def test_with_run_command_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_with_run_command()

    @add_test_categories(["msvcstl"])
    def test_with_run_command_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test_with_run_command()



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CPPAcceleratorTableTestCase(TestBase):
    @skipUnlessDarwin
    @skipIf(debug_info=no_match(["dwarf"]))
    @skipIf(dwarf_version=[">=", "5"])
    def test(self):
        """Test that type lookups fail early (performance)"""
        self.build()

        logfile = self.getBuildArtifact("dwarf.log")

        self.expect("log enable dwarf lookups -f" + logfile)
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp")
        )
        # Pick one from the middle of the list to have a high chance
        # of it not being in the first file looked at.
        self.expect("frame variable inner_d")

        with open(logfile) as f:
            log = f.readlines()
        n = 0
        for line in log:
            if re.findall(r"[abcdefg]\.o: FindByNameAndTag\(\)", line):
                self.assertIn("d.o", line)
                n += 1

        self.assertEqual(n, 1, "".join(log))



# RUN: %PYTHON %s | FileCheck %s

from mlir import ir
from mlir.dialects import transform
from mlir.dialects.transform import tune, debug


def run(f):
    print("\n// TEST:", f.__name__)
    with ir.Context(), ir.Location.unknown():
        module = ir.Module.create()
        with ir.InsertionPoint(module.body):
            sequence = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [],
                transform.AnyOpType.get(),
            )
            with ir.InsertionPoint(sequence.body):
                f(sequence.bodyTarget)
                transform.YieldOp()
        print(module)
    return f


# CHECK-LABEL: TEST: testKnobOp
@run
def testKnobOp(target):
    any_param = transform.AnyParamType.get()

    # CHECK: %[[HEADS_OR_TAILS:.*]] = transform.tune.knob<"coin"> options = [true, false] -> !transform.any_param
    heads_or_tails = tune.KnobOp(
        result=any_param, name=ir.StringAttr.get("coin"), options=[True, False]
    )
    # CHECK: transform.tune.knob<"animal"> options = ["cat", "dog", unit] -> !transform.any_param
    tune.KnobOp(any_param, name="animal", options=["cat", "dog", ir.UnitAttr.get()])
    # CHECK: transform.tune.knob<"tile_size"> options = [2, 4, 8, 16, 24, 32] -> !transform.any_param
    tune.KnobOp(any_param, "tile_size", [2, 4, 8, 16, 24, 32])
    # CHECK: transform.tune.knob<"magic_value"> options = [2.000000e+00, 2.250000e+00, 2.500000e+00, 2.750000e+00, 3.000000e+00] -> !transform.any_param
    tune.knob(any_param, "magic_value", [2.0, 2.25, 2.5, 2.75, 3.0])

    # CHECK: transform.debug.emit_param_as_remark %[[HEADS_OR_TAILS]]
    debug.emit_param_as_remark(heads_or_tails)

    # CHECK: %[[HEADS:.*]] = transform.tune.knob<"coin"> = true from options = [true, false] -> !transform.any_param
    heads = tune.KnobOp(any_param, "coin", options=[True, False], selected=True)
    # CHECK: transform.tune.knob<"animal"> = "dog" from options = ["cat", "dog", unit] -> !transform.any_param
    tune.KnobOp(
        any_param,
        name="animal",
        options=["cat", "dog", ir.UnitAttr.get()],
        selected="dog",
    )
    # CHECK: transform.tune.knob<"tile_size"> = 8 : i64 from options = [2, 4, 8, 16, 24, 32] -> !transform.any_param
    tune.KnobOp(any_param, "tile_size", [2, 4, 8, 16, 24, 32], selected=8)
    # CHECK: transform.tune.knob<"magic_value"> = 2.500000e+00 : f64 from options = [2.000000e+00, 2.250000e+00, 2.500000e+00, 2.750000e+00, 3.000000e+00] -> !transform.any_param
    tune.knob(any_param, "magic_value", [2.0, 2.25, 2.5, 2.75, 3.0], selected=2.5)

    # CHECK: transform.debug.emit_param_as_remark %[[HEADS]]
    debug.emit_param_as_remark(heads)

    # CHECK: transform.tune.knob<"range_as_a_dict"> = 4 : i64 from options = {start = 2 : i64, step = 2 : i64, stop = 16 : i64} -> !transform.any_param
    # NB: Membership of `selected` in non-ArrayAttr `options` is _not_ verified.
    i64 = ir.IntegerType.get_signless(64)
    tune.knob(
        any_param,
        "range_as_a_dict",
        ir.DictAttr.get(
            {
                "start": ir.IntegerAttr.get(i64, 2),
                "stop": ir.IntegerAttr.get(i64, 16),
                "step": ir.IntegerAttr.get(i64, 2),
            }
        ),
        selected=4,
    )


# CHECK-LABEL: TEST: testAlternativesOp
@run
def testAlternativesOp(target):
    any_param = transform.AnyParamType.get()

    # CHECK: %[[LEFT_OR_RIGHT_OUTCOME:.*]] = transform.tune.alternatives<"left_or_right"> -> !transform.any_param {
    left_or_right = tune.AlternativesOp(
        [transform.AnyParamType.get()], "left_or_right", 2
    )
    idx_for_left, idx_for_right = 0, 1
    with ir.InsertionPoint(left_or_right.alternatives[idx_for_left].blocks[0]):
        # CHECK: %[[C0:.*]] = transform.param.constant 0
        i32_0 = ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 0)
        c0 = transform.ParamConstantOp(transform.AnyParamType.get(), i32_0)
        # CHECK: transform.yield %[[C0]]
        transform.yield_(c0)
    # CHECK-NEXT: }, {
    with ir.InsertionPoint(left_or_right.alternatives[idx_for_right].blocks[0]):
        # CHECK: %[[C1:.*]] = transform.param.constant 1
        i32_1 = ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 1)
        c1 = transform.ParamConstantOp(transform.AnyParamType.get(), i32_1)
        # CHECK: transform.yield %[[C1]]
        transform.yield_(c1)
    # CHECK-NEXT: }
    outcome_of_left_or_right_decision = left_or_right.results[0]

    # CHECK: transform.tune.alternatives<"fork_in_the_road"> selected_region = 0 -> !transform.any_param {
    fork_in_the_road = tune.AlternativesOp(
        [transform.AnyParamType.get()], "fork_in_the_road", 2, selected_region=0
    )
    with ir.InsertionPoint(fork_in_the_road.alternatives[idx_for_left].blocks[0]):
        # CHECK: %[[C0:.*]] = transform.param.constant 0
        i32_0 = ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 0)
        c0 = transform.ParamConstantOp(transform.AnyParamType.get(), i32_0)
        # CHECK: transform.yield %[[C0]]
        transform.yield_(c0)
    # CHECK-NEXT: }, {
    with ir.InsertionPoint(fork_in_the_road.alternatives[idx_for_right].blocks[0]):
        # CHECK: %[[C1:.*]] = transform.param.constant 1
        i32_1 = ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 1)
        c1 = transform.ParamConstantOp(transform.AnyParamType.get(), i32_1)
        # CHECK: transform.yield %[[C1]]
        transform.yield_(c1)
    # CHECK-NEXT: }

    # CHECK: transform.tune.alternatives<"left_or_right_as_before"> selected_region = %[[LEFT_OR_RIGHT_OUTCOME]] : !transform.any_param {
    left_or_right_as_before = tune.AlternativesOp(
        [],
        "left_or_right_as_before",
        2,
        selected_region=outcome_of_left_or_right_decision,
    )
    with ir.InsertionPoint(
        left_or_right_as_before.alternatives[idx_for_left].blocks[0]
    ):
        # CHECK: transform.param.constant 1337
        i32_1337 = ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 1337)
        c1337 = transform.ParamConstantOp(transform.AnyParamType.get(), i32_1337)
        # CHECK: transform.debug.emit_param_as_remark
        debug.emit_param_as_remark(c1337)
        transform.yield_([])
    # CHECK-NEXT: }, {
    with ir.InsertionPoint(
        left_or_right_as_before.alternatives[idx_for_right].blocks[0]
    ):
        # CHECK: transform.param.constant 42
        i32_42 = ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 42)
        c42 = transform.ParamConstantOp(transform.AnyParamType.get(), i32_42)
        # CHECK: transform.debug.emit_param_as_remark
        debug.emit_param_as_remark(c42)
        transform.yield_([])
    # CHECK-NEXT: }



import lldbsuite.test.lldbinline as lldbinline
from lldbsuite.test.decorators import *

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    [
        expectedFailureAll(oslist=["linux"], bugnumber="llvm.org/pr36107"),
        add_test_categories(["gmodules"]),
    ],
)



"""
Specification, assembler, disassembler, and interpreter
for LLDB dataformatter bytecode.

See https://lldb.llvm.org/resources/formatterbytecode.html for more details.
"""

from __future__ import annotations

# Work around the fact that one of the local files is called
# types.py, which breaks some versions of python.
import os, sys

path = os.path.abspath(os.path.dirname(__file__))
if path in sys.path:
    sys.path.remove(path)

import re
import io
import ast
import enum
import shlex
import textwrap
from copy import copy
from dataclasses import dataclass
from typing import Any, BinaryIO, Optional, Sequence, TextIO, Tuple, Union, cast

BINARY_VERSION = 1

# Types
type_String = 1
type_Int = 2
type_UInt = 3
type_Object = 4
type_Type = 5

# Opcodes
opcode = dict()


def define_opcode(n, mnemonic, name):
    globals()["op_" + name] = n
    if mnemonic:
        opcode[mnemonic] = n
    opcode[n] = mnemonic


define_opcode(1, "dup", "dup")
define_opcode(2, "drop", "drop")
define_opcode(3, "pick", "pick")
define_opcode(4, "over", "over")
define_opcode(5, "swap", "swap")
define_opcode(6, "rot", "rot")

define_opcode(0x10, "{", "begin")
define_opcode(0x11, "if", "if")
define_opcode(0x12, "ifelse", "ifelse")
define_opcode(0x13, "return", "return")

define_opcode(0x20, None, "lit_uint")
define_opcode(0x21, None, "lit_int")
define_opcode(0x22, None, "lit_string")
define_opcode(0x23, None, "lit_selector")

define_opcode(0x2A, "as_int", "as_int")
define_opcode(0x2B, "as_uint", "as_uint")
define_opcode(0x2C, "is_null", "is_null")

define_opcode(0x30, "+", "plus")
define_opcode(0x31, "-", "minus")
define_opcode(0x32, "*", "mul")
define_opcode(0x33, "/", "div")
define_opcode(0x34, "%", "mod")
define_opcode(0x35, "<<", "shl")
define_opcode(0x36, ">>", "shr")

define_opcode(0x40, "&", "and")
define_opcode(0x41, "|", "or")
define_opcode(0x42, "^", "xor")
define_opcode(0x43, "~", "not")

define_opcode(0x50, "=", "eq")
define_opcode(0x51, "!=", "neq")
define_opcode(0x52, "<", "lt")
define_opcode(0x53, ">", "gt")
define_opcode(0x54, "=<", "le")
define_opcode(0x55, ">=", "ge")

define_opcode(0x60, "call", "call")

# Function signatures
sig_summary = 0
sig_init = 1
sig_get_num_children = 2
sig_get_child_index = 3
sig_get_child_at_index = 4
sig_get_value = 5
sig_update = 6

SIGNATURES = {
    "summary": sig_summary,
    "init": sig_init,
    "get_num_children": sig_get_num_children,
    "get_child_index": sig_get_child_index,
    "get_child_at_index": sig_get_child_at_index,
    "get_value": sig_get_value,
    "update": sig_update,
}

SIGNATURE_NAMES = "|".join(SIGNATURES.keys())
SIGNATURE_IDS = {v: k for k, v in SIGNATURES.items()}

# Selectors
selector = dict()


def define_selector(n, name):
    globals()["sel_" + name] = n
    selector["@" + name] = n
    selector[n] = "@" + name


define_selector(0, "summary")
define_selector(1, "type_summary")

define_selector(0x10, "get_num_children")
define_selector(0x11, "get_child_at_index")
define_selector(0x12, "get_child_with_name")
define_selector(0x13, "get_child_index")
define_selector(0x15, "get_type")
define_selector(0x16, "get_template_argument_type")
define_selector(0x17, "cast")
define_selector(0x18, "get_synthetic_value")
define_selector(0x19, "get_non_synthetic_value")
define_selector(0x20, "get_value")
define_selector(0x21, "get_value_as_unsigned")
define_selector(0x22, "get_value_as_signed")
define_selector(0x23, "get_value_as_address")

define_selector(0x40, "read_memory_byte")
define_selector(0x41, "read_memory_uint32")
define_selector(0x42, "read_memory_int32")
define_selector(0x43, "read_memory_unsigned")
define_selector(0x44, "read_memory_signed")
define_selector(0x45, "read_memory_address")
define_selector(0x46, "read_memory")

define_selector(0x50, "fmt")
define_selector(0x51, "sprintf")
define_selector(0x52, "strlen")


################################################################################
# Assembler.
################################################################################

_SIGNATURE_LABEL = re.compile(f"@(?:{SIGNATURE_NAMES}):$")


def _tokenize(assembler: str) -> list[str]:
    """Convert string of assembly into tokens."""
    # With one exception, tokens are sequences of non-space characters.
    # The one exception is string literals, which may have spaces.

    # To parse strings, which can contain escaped contents, use a "Friedl
    # unrolled loop". The high level of such a regex is:
    #     open normal* ( special normal* )* close
    # which for string literals is:
    string_literal = r'" [^"\\]* (?: \\. [^"\\]* )* "'

    return re.findall(rf"{string_literal} | \S+", assembler, re.VERBOSE)


def _segment_by_signature(input: list[str]) -> list[Tuple[str, list[str]]]:
    """Segment the input tokens along signature labels."""
    segments = []

    # Loop state
    signature = None
    tokens = []

    for token in input:
        if _SIGNATURE_LABEL.match(token):
            if signature:
                segments.append((signature, tokens))
            signature = token[1:-1]  # strip leading @, trailing :
            tokens = []
        else:
            tokens.append(token)

    if signature:
        segments.append((signature, tokens))

    return segments


@dataclass
class BytecodeSection:
    """Abstraction of the data serialized to __lldbformatters sections."""

    type_name: str
    flags: int
    signatures: list[Tuple[str, bytes]]

    def validate(self):
        seen = set()
        for sig, _ in self.signatures:
            if sig in seen:
                raise ValueError(f"duplicate signature: {sig}")
            seen.add(sig)

    def _to_binary(self) -> bytes:
        bin = bytearray()
        bin.extend(_to_uleb(len(self.type_name)))
        bin.extend(bytes(self.type_name, encoding="utf-8"))
        bin.extend(_to_byte(self.flags))
        for sig, bc in self.signatures:
            bin.extend(_to_byte(SIGNATURES[sig]))
            bin.extend(_to_uleb(len(bc)))
            bin.extend(bc)

        return bytes(bin)

    def write_binary(self, output: BinaryIO) -> None:
        self.validate()

        bin = self._to_binary()
        output.write(_to_byte(BINARY_VERSION))
        output.write(_to_uleb(len(bin)))
        output.write(self._to_binary())

    def write_source(self, output: TextIO, language: str) -> None:
        if language == "c":
            self.write_c(output)
        elif language == "swift":
            self.write_swift(output)

    class _CBuilder:
        """Helper class for emitting binary data as a C-string literal."""

        entries: list[Tuple[str, str]]

        def __init__(self) -> None:
            self.entries = []

        def emit_byte(self, x: int, comment: str) -> None:
            self.emit_bytes(_to_byte(x), comment)

        def emit_uleb(self, x: int, comment: str) -> None:
            self.emit_bytes(_to_uleb(x), comment)

        def emit_bytes(self, x: bytes, comment: str) -> None:
            # Construct zero pemited hex values with length two.
            string = "".join(f"\\x{b:02x}" for b in x)
            self.emit_string(string, comment)

        def emit_string(self, string: str, comment: str) -> None:
            self.entries.append((f'"{string}"', comment))

    class _SwiftBuilder:
        """Helper class for emitting binary data as a Swift tuple literal."""

        entries: list[Tuple[bytes, str]]

        def __init__(self) -> None:
            self.entries = []

        def emit_byte(self, x: int, comment: str) -> None:
            self.emit_bytes(_to_byte(x), comment)

        def emit_uleb(self, x: int, comment: str) -> None:
            self.emit_bytes(_to_uleb(x), comment)

        def emit_bytes(self, x: bytes, comment: str) -> None:
            self.entries.append((x, comment))

        def emit_string(self, string: str, comment: str) -> None:
            self.emit_bytes(string.encode(), comment)

        @property
        def type_decl(self):
            total_bytes = sum((len(bs) for bs, _ in self.entries))
            element_list = ", ".join(["UInt8"] * total_bytes)
            return f"({element_list})"

    def _build(self, builder) -> None:
        size = len(self._to_binary())
        builder.emit_byte(BINARY_VERSION, "version")
        builder.emit_uleb(size, "remaining record size")
        builder.emit_uleb(len(self.type_name), "type name size")
        builder.emit_string(self.type_name, "type name")
        builder.emit_byte(self.flags, "flags")
        for sig, bc in self.signatures:
            builder.emit_byte(SIGNATURES[sig], f"sig_{sig}")
            builder.emit_uleb(len(bc), "program size")
            builder.emit_bytes(bc, "program")

    def write_c(self, output: TextIO) -> None:
        self.validate()

        builder = self._CBuilder()
        self._build(builder)

        print(
            textwrap.dedent(
                """
                #ifdef __APPLE__
                #define FORMATTER_SECTION "__DATA_CONST,__lldbformatters"
                #else
                #define FORMATTER_SECTION ".lldbformatters"
                #endif
                """
            ),
            file=output,
        )
        print(
            "__attribute__((used, section(FORMATTER_SECTION)))",
            file=output,
        )
        var_name = re.sub(r"\W", "_", self.type_name)
        print(f"unsigned char _{var_name}_formatter[] =", file=output)
        indent = "    "
        for string, comment in builder.entries:
            print(f"{indent}// {comment}", file=output)
            print(f"{indent}{string}", file=output)
        print(";", file=output)

    def write_swift(self, output: TextIO) -> None:
        self.validate()

        builder = self._SwiftBuilder()
        self._build(builder)

        print(
            textwrap.dedent(
                """\
                #if swift(>=6.3)
                #if objectFormat(MachO)
                @section("__DATA_CONST,__lldbformatters")
                #else
                @section(".lldbformatters")
                #endif
                @used"""
            ),
            file=output,
        )
        print(
            f"let `{self.type_name} formatter`: {builder.type_decl} = (",
            file=output,
        )
        indent = "    "
        for bs, comment in builder.entries:
            print(f"{indent}// {comment}", file=output)
            byte_list = ", ".join(f"0x{b:02x}" for b in bs)
            print(f"{indent}{byte_list},", file=output)
        print(")", file=output)
        print("#endif", file=output)  # swift(>=6.3)


def assemble_file(type_name: str, input: TextIO) -> BytecodeSection:
    input_tokens = _tokenize(input.read())
    signatures = []
    for sig, tokens in _segment_by_signature(input_tokens):
        if tokens:
            signatures.append((sig, assemble_tokens(tokens)))

    return BytecodeSection(type_name, flags=0, signatures=signatures)


def assemble(assembly: str) -> bytes:
    return assemble_tokens(_tokenize(assembly))


def assemble_tokens(tokens: list[str]) -> bytes:
    """Assemble assembly into bytecode"""
    # This is a stack of all in-flight/unterminated blocks.
    bytecode = [bytearray()]

    def emit(byte):
        bytecode[-1].append(byte)

    tokens.reverse()
    while tokens:
        tok = tokens.pop()
        if tok == "":
            pass
        elif tok == "{":
            bytecode.append(bytearray())
        elif tok == "}":
            block = bytecode.pop()
            emit(op_begin)
            emit(len(block))  # FIXME: uleb
            bytecode[-1].extend(block)
        elif tok[0].isdigit():
            if tok[-1] == "u":
                emit(op_lit_uint)
                emit(int(tok[:-1]))  # FIXME
            else:
                emit(op_lit_int)
                emit(int(tok))  # FIXME
        elif tok[0] == "@":
            emit(op_lit_selector)
            emit(selector[tok])
        elif tok[0] == '"':
            # Remove backslash escaping '"' and '\'.
            s = re.sub(r'\\(["\\])', r"\1", tok[1:-1]).encode()
            emit(op_lit_string)
            emit(len(s))
            bytecode[-1].extend(s)
        else:
            emit(opcode[tok])
    assert len(bytecode) == 1  # unterminated {
    return bytes(bytecode[0])


################################################################################
# Disassembler.
################################################################################


def disassemble_file(input: BinaryIO, output: TextIO) -> None:
    stream = io.BytesIO(input.read())

    version = stream.read(1)[0]
    if version != BINARY_VERSION:
        raise ValueError(f"unknown binary version: {version}")

    record_size = _from_uleb(stream)
    stream.truncate(stream.tell() + record_size)

    name_size = _from_uleb(stream)
    _type_name = stream.read(name_size).decode()
    _flags = stream.read(1)[0]

    while True:
        sig_byte = stream.read(1)
        if not sig_byte:
            break
        sig_name = SIGNATURE_IDS[sig_byte[0]]
        body_size = _from_uleb(stream)
        bc = stream.read(body_size)
        asm, _ = disassemble(bc)
        print(f"@{sig_name}: {asm}", file=output)


def disassemble(bytecode: bytes) -> Tuple[str, list[int]]:
    """Disassemble bytecode into (assembly, token starts)"""
    asm = ""
    all_bytes = list(bytecode)
    all_bytes.reverse()
    blocks = []
    tokens = [0]

    def next_byte():
        """Fetch the next byte in the bytecode and keep track of all
        in-flight blocks"""
        for i in range(len(blocks)):
            blocks[i] -= 1
        tokens.append(len(asm))
        return all_bytes.pop()

    while all_bytes:
        b = next_byte()
        if b == op_begin:
            asm += "{"
            length = next_byte()
            blocks.append(length)
        elif b == op_lit_uint:
            b = next_byte()
            asm += str(b)  # FIXME uleb
            asm += "u"
        elif b == op_lit_int:
            b = next_byte()
            asm += str(b)
        elif b == op_lit_selector:
            b = next_byte()
            asm += selector[b]
        elif b == op_lit_string:
            length = next_byte()
            s = '"'
            for _ in range(length):
                c = chr(next_byte())
                if c in ('"', "\\"):
                    s += "\\"
                s += c
            s += '"'
            asm += s
        else:
            asm += opcode[b]

        while blocks and blocks[-1] == 0:
            asm += " }"
            blocks.pop()

        if all_bytes:
            asm += " "

    if blocks:
        asm += "ERROR"
    return asm, tokens


################################################################################
# Interpreter.
################################################################################


def count_fmt_params(fmt: str) -> int:
    """Count the number of parameters in a format string"""
    from string import Formatter

    f = Formatter()
    n = 0
    for _, name, _, _ in f.parse(fmt):
        if name > n:
            n = name
    return n


def interpret(bytecode: bytes, control: list, data: list, tracing: bool = False):
    """Interpret bytecode"""
    frame = []
    frame.append((0, len(bytecode)))

    def trace():
        """print a trace of the execution for debugging purposes"""

        def fmt(d):
            if isinstance(d, int):
                return str(d)
            if isinstance(d, str):
                return d
            return repr(type(d))

        pc, end = frame[-1]
        asm, tokens = disassemble(bytecode)
        print(
            "=== frame = {1}, data = {2}, opcode = {0}".format(
                opcode[b], frame, [fmt(d) for d in data]
            )
        )
        print(asm)
        print(" " * (tokens[pc]) + "^")

    def next_byte():
        """Fetch the next byte and update the PC"""
        pc, end = frame[-1]
        assert pc < len(bytecode)
        b = bytecode[pc]
        frame[-1] = pc + 1, end
        # At the end of a block?
        while pc >= end:
            frame.pop()
            if not frame:
                return None
            pc, end = frame[-1]
            if pc >= end:
                return None
            b = bytecode[pc]
            frame[-1] = pc + 1, end
        return b

    while frame[-1][0] < len(bytecode):
        b = next_byte()
        if b == None:
            break
        if tracing:
            trace()
        # Data stack manipulation.
        if b == op_dup:
            data.append(data[-1])
        elif b == op_drop:
            data.pop()
        elif b == op_pick:
            data.append(data[data.pop()])
        elif b == op_over:
            data.append(data[-2])
        elif b == op_swap:
            x = data.pop()
            y = data.pop()
            data.append(x)
            data.append(y)
        elif b == op_rot:
            z = data.pop()
            y = data.pop()
            x = data.pop()
            data.append(z)
            data.append(x)
            data.append(y)

        # Control stack manipulation.
        elif b == op_begin:
            length = next_byte()
            pc, end = frame[-1]
            control.append((pc, pc + length))
            frame[-1] = pc + length, end
        elif b == op_if:
            if data.pop():
                frame.append(control.pop())
        elif b == op_ifelse:
            if data.pop():
                control.pop()
                frame.append(control.pop())
            else:
                frame.append(control.pop())
                control.pop()
        elif b == op_return:
            control.clear()
            return data[-1]

        # Literals.
        elif b == op_lit_uint:
            b = next_byte()  # FIXME uleb
            data.append(int(b))
        elif b == op_lit_int:
            b = next_byte()  # FIXME uleb
            data.append(int(b))
        elif b == op_lit_selector:
            b = next_byte()
            data.append(b)
        elif b == op_lit_string:
            length = next_byte()
            s = ""
            while length:
                s += chr(next_byte())
                length -= 1
            data.append(s)

        elif b == op_as_uint:
            pass
        elif b == op_as_int:
            pass
        elif b == op_is_null:
            data.append(1 if data.pop() == None else 0)

        # Arithmetic, logic, etc.
        elif b == op_plus:
            data.append(data.pop() + data.pop())
        elif b == op_minus:
            data.append(-data.pop() + data.pop())
        elif b == op_mul:
            data.append(data.pop() * data.pop())
        elif b == op_div:
            y = data.pop()
            data.append(data.pop() / y)
        elif b == op_mod:
            y = data.pop()
            data.append(data.pop() % y)
        elif b == op_shl:
            y = data.pop()
            data.append(data.pop() << y)
        elif b == op_shr:
            y = data.pop()
            data.append(data.pop() >> y)
        elif b == op_and:
            data.append(data.pop() & data.pop())
        elif b == op_or:
            data.append(data.pop() | data.pop())
        elif b == op_xor:
            data.append(data.pop() ^ data.pop())
        elif b == op_not:
            data.append(not data.pop())
        elif b == op_eq:
            data.append(data.pop() == data.pop())
        elif b == op_neq:
            data.append(data.pop() != data.pop())
        elif b == op_lt:
            data.append(data.pop() > data.pop())
        elif b == op_gt:
            data.append(data.pop() < data.pop())
        elif b == op_le:
            data.append(data.pop() >= data.pop())
        elif b == op_ge:
            data.append(data.pop() <= data.pop())

        # Function calls.
        elif b == op_call:
            sel = data.pop()
            if sel == sel_summary:
                data.append(data.pop().GetSummary())
            elif sel == sel_get_num_children:
                data.append(data.pop().GetNumChildren())
            elif sel == sel_get_child_at_index:
                index = data.pop()
                valobj = data.pop()
                data.append(valobj.GetChildAtIndex(index))
            elif sel == sel_get_child_with_name:
                name = data.pop()
                valobj = data.pop()
                data.append(valobj.GetChildMemberWithName(name))
            elif sel == sel_get_child_index:
                name = data.pop()
                valobj = data.pop()
                data.append(valobj.GetIndexOfChildWithName(name))
            elif sel == sel_get_type:
                data.append(data.pop().GetType())
            elif sel == sel_get_template_argument_type:
                n = data.pop()
                valobj = data.pop()
                data.append(valobj.GetTemplateArgumentType(n))
            elif sel == sel_get_synthetic_value:
                data.append(data.pop().GetSyntheticValue())
            elif sel == sel_get_non_synthetic_value:
                data.append(data.pop().GetNonSyntheticValue())
            elif sel == sel_get_value:
                data.append(data.pop().GetValue())
            elif sel == sel_get_value_as_unsigned:
                data.append(data.pop().GetValueAsUnsigned())
            elif sel == sel_get_value_as_signed:
                data.append(data.pop().GetValueAsSigned())
            elif sel == sel_get_value_as_address:
                data.append(data.pop().GetValueAsAddress())
            elif sel == sel_cast:
                sbtype = data.pop()
                valobj = data.pop()
                data.append(valobj.Cast(sbtype))
            elif sel == sel_strlen:
                s = data.pop()
                data.append(len(s) if s else 0)
            elif sel == sel_fmt:
                fmt = data.pop()
                n = count_fmt_params(fmt)
                args = []
                for i in range(n):
                    args.append(data.pop())
                data.append(fmt.format(*args))
            else:
                print("not implemented: " + selector[sel])
                assert False
    return data[-1]


################################################################################
# Python -> Bytecode Compiler
################################################################################

_BUILTINS = {
    "Cast": "@cast",
    "GetChildAtIndex": "@get_child_at_index",
    "GetChildMemberWithName": "@get_child_with_name",
    "GetSummary": "@summary",
    "GetSyntheticValue": "@get_synthetic_value",
    "GetTemplateArgumentType": "@get_template_argument_type",
    "GetType": "@get_type",
    "GetValueAsUnsigned": "@get_value_as_unsigned",
}

_COMPS = {
    ast.Eq: "=",
    ast.NotEq: "!=",
    ast.Lt: "<",
    ast.LtE: "=<",
    ast.Gt: ">",
    ast.GtE: "=>",
}

# Maps Python method names in a formatter class to their bytecode signatures.
_METHOD_SIGS = {
    "__init__": "@init",
    "update": "@update",
    "num_children": "@get_num_children",
    "get_child_index": "@get_child_index",
    "get_child_at_index": "@get_child_at_index",
    "get_value": "@get_value",
}


class CompilerError(Exception):
    lineno: int

    def __init__(self, message, node: Union[ast.expr, ast.stmt]) -> None:
        super().__init__(message)
        self.lineno = node.lineno


class Compiler(ast.NodeVisitor):
    """
    Compile Python LLDB data formatters to LLDB formatter bytecode.

    This compiler is supports a limited subset of Python.

    # Supported Features

    * Top level functions implementing LLDB summary formatters
    * Top level classes implementing LLDB synthetic formatters
    * Partial support for the following, see below for more details:
      - Object attributes (properties)
      - Local variables
      - Function calls
    * Python language support
    [x] If statements (including else, elif and nested if)
    [x] Return statements
    [x] String, integer, float, boolean, and None literals
    [x] Binary comparisons
    [ ] Boolean operators
    [ ] Math operations

    # Unsupported Features

        Note: that this is not exhaustive, refer to the list of supported
        features above.

    * For and while loops
    * Exceptions
    * User defined general purpose functions and classes
    * Lists, dicts, sets, and other container data types
    * Iterators, comprehensions, yield, etc
    * With statements
    * Imports of any modules

    # Variables

    The compiler supports two kinds of variables, local variables and attribute
    variables (properties), but there are limitations on both.

    In __init__ and update, local variables are currently *not* supported, but
    attributes can be assigned to. This matches the common case for these
    functions.

    In all other function bodies, local variables _are_ supported, but
    attributes can only be read from, *not* assigned to. This also matches the
    common case for these functions.

    Variables (local and attributes) are tracked, allowing the compiler to know
    their position in the stack. Variable reads can then be lowered to `pick`
    instructions. See the compiler's `locals` and `attrs` attributes.

    # Functions

    Known functions are supported, a design that customizes the scope of what
    formatters can and can't do. The functions known to the compiler are called
    "selectors". The selectors are primarily SBValue API, although there are
    also general purpose selectors. Formatters can only call selectors, not user
    defined functions, and not SB methods that have not been defined as a
    selector.
    """

    # Names of locals in bottom-to-top stack order. locals[0] is the
    # oldest/deepest; locals[-1] is the most recently pushed.
    locals: list[str]

    # Names of visible attrs in bottom-to-top stack order. Always holds the
    # full combined frame for the method being compiled: grows incrementally
    # during __init__/update, and is set to the combined list before getter
    # methods are compiled.
    attrs: list[str]

    # Bytecode signature of the method being compiled, or None for top-level
    # functions.
    current_sig: Optional[str]

    buffer: io.StringIO

    def __init__(self) -> None:
        self.locals = []
        self.attrs = []
        self.current_sig = None
        self.buffer = io.StringIO()

    def compile(self, source_file: str) -> str:
        with open(source_file) as f:
            root = ast.parse(f.read())
        self.visit(root)
        return self.buffer.getvalue()

    def visit_ClassDef(self, node: ast.ClassDef) -> None:
        # Compile methods in a fixed order so that attrs is fully populated
        # before getter methods are compiled.
        methods = {}
        for item in node.body:
            if isinstance(item, ast.FunctionDef):
                if item.name not in _METHOD_SIGS:
                    raise CompilerError(f"unsupported method: {item.name}", item)
                methods[item.name] = item

        self.attrs = []
        if method := methods.get("__init__"):
            self._compile_method(method)
        # self.attrs now holds init's attrs. update's attrs are appended above
        # them, so after update self.attrs is the combined init+update list.
        if method := methods.get("update"):
            self._compile_method(method)

        for method_name, method in methods.items():
            if method_name not in ("__init__", "update"):
                self._compile_method(method)

    def _compile_method(self, node: ast.FunctionDef) -> None:
        self.current_sig = _METHOD_SIGS[node.name]

        return_type = node.returns.id if isinstance(node.returns, ast.Name) else None
        if node.name == "update" and return_type != "bool":
            raise CompilerError(
                "update must be declared to return bool: def update(self) -> bool:",
                node,
            )

        # Strip 'self' (and 'internal_dict' for __init__) from the arg list;
        # the remaining args become the initial locals.
        args = copy(node.args.args)
        args.pop(0)  # drop 'self'
        if node.name == "__init__":
            args.pop()  # drop trailing 'internal_dict'

        self.locals = [arg.arg for arg in args]

        # Compile into a temporary buffer so the signature line can be
        # emitted first.
        saved_buffer = self.buffer
        self.buffer = io.StringIO()

        self._visit_each(node.body)

        method_output = self.buffer.getvalue()
        self.buffer = saved_buffer
        self._output(f"{self.current_sig}:")
        self._output(method_output)

        self.locals.clear()
        self.current_sig = None

    def visit_FunctionDef(self, node: ast.FunctionDef) -> None:
        # Top-level function (not inside a class).
        self.current_sig = None
        self.attrs = []
        self.locals = [arg.arg for arg in node.args.args]
        self._visit_each(node.body)
        self.locals.clear()

    def visit_Compare(self, node: ast.Compare) -> None:
        self.visit(node.left)
        # XXX: Does not handle multiple comparisons, ex: `0 < x < 10`
        self.visit(node.comparators[0])
        self._output(_COMPS[type(node.ops[0])])

    def visit_If(self, node: ast.If) -> None:
        self.visit(node.test)

        self._output("{")
        self._visit_each(node.body)
        if node.orelse:
            self._output("} {")
            self._visit_each(node.orelse)
            self._output("} ifelse")
        else:
            self._output("} if")

    def visit_Return(self, node: ast.Return) -> None:
        if node.value:
            self.visit(node.value)
        self._output("return")

    def visit_Constant(self, node: ast.Constant) -> None:
        if isinstance(node.value, str):
            self._output(f'"{node.value}"')
        elif isinstance(node.value, bool):
            self._output(int(node.value))
        else:
            self._output(node.value)

    def visit_Call(self, node: ast.Call) -> None:
        func = node.func
        if isinstance(func, ast.Attribute):
            receiver = func.value
            method = func.attr
            # self is not a valid call receiver.
            if isinstance(receiver, ast.Name) and receiver.id == "self":
                raise CompilerError(
                    "self is not a valid call receiver; use self.attr to read an attribute",
                    node,
                )
            if selector := _BUILTINS.get(method):
                self.visit(receiver)
                self._visit_each(node.args)
                self._output(f"{selector} call")
                return
            raise CompilerError(f"unsupported method: {method}", node)

        if isinstance(func, ast.Name):
            raise CompilerError(f"unsupported function: {func.id}", node)

        raise CompilerError("unsupported function call expression", node)

    def visit_Assign(self, node: ast.Assign) -> None:
        target = node.targets[0]

        # Handle self.attr = expr (attribute assignment).
        if (
            isinstance(target, ast.Attribute)
            and isinstance(target.value, ast.Name)
            and target.value.id == "self"
        ):
            if self.current_sig not in ("@init", "@update"):
                raise CompilerError(
                    "attribute assignment is only allowed in __init__ and update",
                    node,
                )

            attr = target.attr
            if attr in self.attrs:
                raise CompilerError(f"attribute '{attr}' is already assigned", node)

            # If the RHS is an argument (the only kind of local permitted in
            # __init__) - then it is already on the stack in place, and no
            # evaluation is needed.
            is_arg = (
                isinstance(node.value, ast.Name)
                and self._local_index(node.value) is not None
            )
            if not is_arg:
                # Evaluate the RHS, leaving its value on the stack.
                self.visit(node.value)

            # Record the attr.
            self.attrs.append(attr)
            return

        # Handle local variable assignment.
        if self.current_sig in ("@init", "@update"):
            raise CompilerError(
                "local variable assignment is not allowed in __init__ or update; "
                "use attribute assignment (self.attr = ...) instead",
                node,
            )

        if isinstance(target, ast.Name):
            names = [target]
        elif isinstance(target, ast.Tuple):
            names = cast(list[ast.Name], target.elts)
        else:
            raise CompilerError("unsupported assignment target", node)

        # Visit RHS, leaving its value on the stack.
        self.visit(node.value)

        # Forget any previous bindings of these names.
        # Their values are orphaned on the stack.
        for name in names:
            idx = self._local_index(name)
            if idx is not None:
                self.locals[idx] = ""

        self.locals.extend(x.id for x in names)

    def visit_Attribute(self, node: ast.Attribute) -> None:
        # Only self.attr reads are supported here.
        if not (isinstance(node.value, ast.Name) and node.value.id == "self"):
            raise CompilerError(
                "unsupported attribute access (only self.attr is supported)", node
            )
        pick_idx = self._attr_index(node.attr, node)
        self._output(f"{pick_idx}u pick")  # "# self.{node.attr}"

    def visit_Name(self, node: ast.Name) -> None:
        idx = self._local_index(node)
        if idx is None:
            raise CompilerError(f"unknown local variable: {node.id}", node)
        self._output(f"{idx}u pick")  # "# {node.id}"

    def _visit_each(self, nodes: Sequence[ast.AST]) -> None:
        for child in nodes:
            self.visit(child)

    def _attr_index(self, name: str, node: ast.expr) -> int:
        # self.attrs is always the full visible attr frame, so the index is
        # the direct pick offset with no further adjustment.
        try:
            return self.attrs.index(name)
        except ValueError:
            raise CompilerError(f"unknown attribute: {name}", node)

    def _local_index(self, name: ast.Name) -> Optional[int]:
        try:
            idx = self.locals.index(name.id)
            # Offset past all attrs.
            return len(self.attrs) + idx
        except ValueError:
            return None

    def _output(self, x: Any) -> None:
        print(x, file=self.buffer)


################################################################################
# Helper functions.
################################################################################


def _to_uleb(value: int) -> bytes:
    """Encode an integer to ULEB128 bytes."""
    if value < 0:
        raise ValueError(f"negative number cannot be encoded to ULEB128: {value}")

    result = bytearray()
    while True:
        byte = value & 0x7F
        value >>= 7
        if value != 0:
            byte |= 0x80
        result.append(byte)
        if value == 0:
            break

    return bytes(result)


def _from_uleb(stream: BinaryIO) -> int:
    """Decode a ULEB128 integer by reading bytes from the stream."""
    result = 0
    shift = 0
    while True:
        byte = stream.read(1)[0]
        result |= (byte & 0x7F) << shift
        shift += 7
        if not (byte & 0x80):
            break

    return result


def _to_byte(n: int) -> bytes:
    return n.to_bytes(1, "big")


def _main():
    import argparse

    parser = argparse.ArgumentParser(
        description="""
    Assembler, disassembler, and interpreter for LLDB dataformatter bytecode.
    See https://lldb.llvm.org/resources/formatterbytecode.html for more details.
    """
    )
    parser.add_argument("input", help="input file")
    mode = parser.add_mutually_exclusive_group()
    mode.add_argument(
        "-c",
        "--compile",
        action="store_true",
        help="compile a Python LLDB data formatter into LLDB formatter bytecode",
    )
    mode.add_argument(
        "-a",
        "--assemble",
        action="store_true",
        help="assemble assembly into bytecode",
    )
    mode.add_argument(
        "-d",
        "--disassemble",
        action="store_true",
        help="disassemble bytecode",
    )
    parser.add_argument("-n", "--type-name", help="source type of formatter")
    parser.add_argument(
        "-o",
        "--output",
        help="output file (required for --assemble)",
    )
    parser.add_argument(
        "--append", action="store_true", help="append to existing output file"
    )
    parser.add_argument(
        "-f",
        "--format",
        choices=("binary", "c", "swift"),
        default="binary",
        help="output file format",
    )
    parser.add_argument("-t", "--test", action="store_true", help="run unit tests")

    args = parser.parse_args()

    if args.append and not args.compile:
        parser.error("--append is valid only with --compile")

    if args.compile:
        if not args.type_name:
            parser.error("--type-name is required with --compile")
        if not args.output:
            parser.error("--output is required with --compile")
        compiler = Compiler()
        try:
            assembly = compiler.compile(args.input)
        except CompilerError as e:
            print(f"{args.input}:{e.lineno}: {e}", file=sys.stderr)
            return

        section = assemble_file(args.type_name, io.StringIO(assembly))
        if args.format == "binary":
            with open(args.output, "wb") as output:
                section.write_binary(output)
        else:
            mode = "a" if args.append else "w"
            with open(args.output, mode) as output:
                section.write_source(output, language=args.format)
    elif args.assemble:
        if not args.type_name:
            parser.error("--type-name is required with --assemble")
        if not args.output:
            parser.error("--output is required with --assemble")
        with open(args.input) as input:
            section = assemble_file(args.type_name, input)
        if args.format == "binary":
            with open(args.output, "wb") as output:
                section.write_binary(output)
        else:
            with open(args.output, "w") as output:
                section.write_source(output, language=args.format)
    elif args.disassemble:
        if args.output:
            with (
                open(args.input, "rb") as input,
                open(args.output, "w") as output,
            ):
                disassemble_file(input, output)
        else:
            with open(args.input, "rb") as input:
                disassemble_file(input, sys.stdout)


if __name__ == "__main__":
    if not ("-t" in sys.argv or "--test" in sys.argv):
        _main()
        sys.exit()

    ############################################################################
    # Tests.
    ############################################################################
    import unittest

    class TestAssembler(unittest.TestCase):

        def test_assemble(self):
            self.assertEqual(assemble("1u dup").hex(), "200101")
            self.assertEqual(assemble('"1u dup"').hex(), "2206317520647570")
            self.assertEqual(assemble("16 < { dup } if").hex(), "21105210010111")
            self.assertEqual(assemble('{ { " } " } }').hex(), "100710052203207d20")

            def roundtrip(asm):
                self.assertEqual(disassemble(assemble(asm))[0], asm)

            roundtrip("1u dup")
            roundtrip("16 < { dup } if")
            roundtrip('{ { " } " } }')

            # String specific checks.
            roundtrip('1u "2u 3u"')
            roundtrip('"a  b"')
            roundtrip('"a \\" b"')

            self.assertEqual(interpret(assemble("1 1 +"), [], []), 2)
            self.assertEqual(interpret(assemble("2 1 1 + *"), [], []), 4)
            self.assertEqual(
                interpret(assemble('2 1 > { "yes" } { "no" } ifelse'), [], []), "yes"
            )

        def test_assemble_file(self):
            def run_assemble(type_name, asm):
                out = io.BytesIO()
                section = assemble_file(type_name, io.StringIO(asm))
                section.write_binary(out)
                out.seek(0)
                return out

            def run_disassemble(binary):
                out = io.StringIO()
                disassemble_file(binary, out)
                out.seek(0)
                return out

            # assemble -> disassemble -> assemble round-trip: binary is identical.
            asm = "@summary: dup @get_value_as_unsigned call return\n@get_num_children: drop 5u return"
            binary1 = run_assemble("MyType", asm)
            dis = run_disassemble(binary1)
            binary2 = run_assemble("MyType", dis.read())
            self.assertEqual(binary1.getvalue(), binary2.getvalue())

            # disassemble -> assemble -> disassemble round-trip: text is identical.
            dis2 = run_disassemble(binary2)
            self.assertEqual(dis.getvalue(), dis2.getvalue())

            # disassemble output contains expected signatures.
            self.assertIn("@summary:", dis.getvalue())
            self.assertIn("@get_num_children:", dis.getvalue())

            # Duplicate signature is an error.
            with self.assertRaises(ValueError):
                run_assemble("MyType", "@summary: 1u return\n@summary: 2u return")

        def test_write_source(self):
            # Use the Account example from main.cpp as a reference, whose
            # exact byte values are known.
            section = BytecodeSection(
                type_name="Account",
                flags=0,
                signatures=[
                    ("get_num_children", bytes([0x20, 0x01])),
                    ("get_child_at_index", bytes([0x02, 0x20, 0x00, 0x23, 0x11, 0x60])),
                ],
            )
            out = io.StringIO()
            section.write_source(out, language="c")
            src = out.getvalue()

            self.assertIn("__attribute__((used, section(FORMATTER_SECTION)))", src)
            self.assertIn("unsigned char _Account_formatter[] =", src)
            self.assertIn('"\\x01"', src)  # version
            self.assertIn('"\\x15"', src)  # record size (21)
            self.assertIn('"\\x07"', src)  # type name size (7)
            self.assertIn('"Account"', src)  # type name
            self.assertIn('"\\x00"', src)  # flags
            self.assertIn('"\\x02"', src)  # sig_get_num_children
            self.assertIn('"\\x20\\x01"', src)  # program
            self.assertIn('"\\x04"', src)  # sig_get_child_at_index
            self.assertIn('"\\x06"', src)  # program size
            self.assertIn('"\\x02\\x20\\x00\\x23\\x11\\x60"', src)  # program
            self.assertIn("// version", src)
            self.assertIn("// type name", src)
            self.assertIn("// program", src)
            # Semicolon terminates the array initializer.
            self.assertEqual(src.count(";"), 1)

            # Non-identifier characters in the type name are replaced with '_'.
            out2 = io.StringIO()
            BytecodeSection("std::vector<int>", 0, []).write_source(out2, language="c")
            self.assertIn("_std__vector_int__formatter[] =", out2.getvalue())

    unittest.main(argv=[__file__])



"""
Test SBProcess APIs, including ReadMemory(), WriteMemory(), and others.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbutil import get_stopped_thread, state_type_to_str


class SignalsAPITestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfWindows  # Windows doesn't have signals
    def test_ignore_signal(self):
        """Test Python SBUnixSignals.Suppress/Stop/Notify() API."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        line = line_number(
            "main.cpp", "// Set break point at this line and setup signal ignores."
        )
        breakpoint = target.BreakpointCreateByLocation("main.cpp", line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        thread = get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(), "There should be a thread stopped due to breakpoint"
        )

        unix_signals = process.GetUnixSignals()
        sigint = unix_signals.GetSignalNumberFromName("SIGINT")
        unix_signals.SetShouldSuppress(sigint, True)
        unix_signals.SetShouldStop(sigint, False)
        unix_signals.SetShouldNotify(sigint, False)

        process.Continue()
        self.assertEqual(
            process.state, lldb.eStateExited, "The process should have exited"
        )
        self.assertEqual(
            process.GetExitStatus(), 0, "The process should have returned 0"
        )



"""
Platform-agnostic helper to query for CPU features.
"""

import re


class CPUFeature:
    def __init__(self, linux_cpu_info_flag: str = None, darwin_sysctl_key: str = None):
        self.cpu_info_flag = linux_cpu_info_flag
        self.sysctl_key = darwin_sysctl_key

    def __str__(self):
        for arch_class in ALL_ARCHS:
            for feat_var in dir(arch_class):
                if self == getattr(arch_class, feat_var):
                    return f"{arch_class.__name__}.{feat_var}"
        raise AssertionError("unreachable")

    def is_supported(self, triple, cmd_runner):
        if re.match(".*-.*-linux", triple):
            err_msg, res = self._is_supported_linux(cmd_runner)
        elif re.match(".*-apple-.*", triple):
            err_msg, res = self._is_supported_darwin(cmd_runner)
        else:
            err_msg, res = None, False

        if err_msg:
            print(f"CPU feature check failed: {err_msg}")

        return res

    def _is_supported_linux(self, cmd_runner):
        if not self.cpu_info_flag:
            return f"Unspecified cpuinfo flag for {self}", False

        cmd = "cat /proc/cpuinfo"
        err, retcode, output = cmd_runner(cmd)
        if err.Fail() or retcode != 0:
            return output, False

        # Assume that every processor presents the same features.
        # Look for the first "Features: ...." line. Features are space separated.
        if m := re.search(r"Features\s*: (.*)\n", output):
            features = m.group(1).split()
            return None, (self.cpu_info_flag in features)

        return 'No "Features:" line found in /proc/cpuinfo', False

    def _is_supported_darwin(self, cmd_runner):
        if not self.sysctl_key:
            return f"Unspecified sysctl key for {self}", False

        cmd = f"sysctl -n {self.sysctl_key}"
        err, retcode, output = cmd_runner(cmd)
        if err.Fail() or retcode != 0:
            return output, False

        return None, (output.strip() == "1")


class AArch64:
    FPMR = CPUFeature("fpmr")
    POE = CPUFeature("poe")
    GCS = CPUFeature("gcs")
    MTE = CPUFeature("mte", "hw.optional.arm.FEAT_MTE4")
    MTE_STORE_ONLY = CPUFeature("mtestoreonly")
    PTR_AUTH = CPUFeature("paca", "hw.optional.arm.FEAT_PAuth2")
    SME = CPUFeature("sme", "hw.optional.arm.FEAT_SME")
    SME_FA64 = CPUFeature("smefa64")
    SME2 = CPUFeature("sme2", "hw.optional.arm.FEAT_SME2")
    SVE = CPUFeature("sve")


class Loong:
    LASX = CPUFeature("lasx")
    LSX = CPUFeature("lsx")


ALL_ARCHS = [AArch64, Loong]



"""
Test the output of `frame diagnose` for dereferencing a function's return value
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDiagnoseDereferenceFunctionReturn(TestBase):
    @expectedFailureAll(oslist=["windows"])
    @skipIf(
        archs=no_match(["x86_64"])
    )  # <rdar://problem/33842388> frame diagnose doesn't work for armv7 or arm64
    @expectedFailureAll(oslist=["macosx"], archs=["i386"], bugnumber="rdar://28656408")
    def test_diagnose_dereference_function_return(self):
        TestBase.setUp(self)
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        self.runCmd("run", RUN_SUCCEEDED)
        self.expect("thread list", "Thread should be stopped", substrs=["stopped"])
        self.expect(
            "frame diagnose", "Crash diagnosis was accurate", substrs=["GetAFoo", "->b"]
        )



"""Test the SBDModule APIs."""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

import os, signal, subprocess


class SBModuleAPICase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.background_pid = None

    def tearDown(self):
        TestBase.tearDown(self)
        if self.background_pid:
            os.kill(self.background_pid, signal.SIGKILL)

    @skipIfRemote
    def test_GetObjectName(self):
        """Test the SBModule::GetObjectName() method"""
        self.build()
        exe = self.getBuildArtifact("a.out")
        libfoo_path = self.getBuildArtifact("libfoo.a")
        target_exe = self.dbg.CreateTarget(exe)
        self.assertTrue(target_exe.IsValid(), "Target for a.out is valid")

        # Test that the executable module has no object name (usually the first module in the target)
        exe_module = target_exe.GetModuleAtIndex(0)
        self.assertTrue(exe_module.IsValid(), "Executable module is valid")
        self.assertIsNone(
            exe_module.GetObjectName(), "a.out should have no object name"
        )

        # check archive member names
        module_specs = lldb.SBModuleSpecList.GetModuleSpecifications(libfoo_path)
        self.assertGreater(
            module_specs.GetSize(), 0, "Archive should have at least one module spec"
        )
        self.assertEqual(
            module_specs[0].GetObjectName(),
            module_specs.GetSpecAtIndex(0).GetObjectName(),
            "subscript [0] matches GetSpecAtIndex(0)",
        )
        self.assertEqual(
            module_specs[-1].GetObjectName(),
            module_specs.GetSpecAtIndex(module_specs.GetSize() - 1).GetObjectName(),
            "subscript [-1] matches last item",
        )
        found = set()
        expected = {"a.o", "b.o"}
        for i in range(module_specs.GetSize()):
            spec = module_specs.GetSpecAtIndex(i)
            obj_name = spec.GetObjectName()
            self.assertIsInstance(obj_name, str)
            self.assertIn(obj_name, expected, f"Unexpected object name: {obj_name}")
            # create a module from the arhive using the sepc
            module = lldb.SBModule(spec)
            self.assertTrue(module.IsValid(), "Module is valid")
            self.assertTrue(module.IsValid(), f"Module for {obj_name} is valid")
            self.assertEqual(
                module.GetObjectName(), obj_name, f"Object name for {obj_name} matches"
            )
            found.add(obj_name)

        self.assertEqual(found, expected, "Did not find all expected archive members")

    @skipUnlessDarwin
    @skipIfRemote
    def test_module_is_file_backed(self):
        """Test the SBModule::IsFileBacked() method"""
        self.build()
        target, _, _, _ = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        self.assertGreater(target.GetNumModules(), 0)
        main_module = target.GetModuleAtIndex(0)
        self.assertEqual(main_module.GetFileSpec().GetFilename(), "a.out")
        self.assertTrue(
            main_module.IsFileBacked(), "The module should be backed by a file on disk"
        )

        self.dbg.DeleteTarget(target)
        self.assertEqual(self.dbg.GetNumTargets(), 0)

        exe = self.getBuildArtifact("a.out")
        background_process = subprocess.Popen([exe])
        self.assertTrue(background_process, "process is not valid")
        self.background_pid = background_process.pid
        os.unlink(exe)

        target = self.dbg.CreateTarget("")
        self.assertEqual(self.dbg.GetNumTargets(), 1)
        error = lldb.SBError()
        process = target.AttachToProcessWithID(
            self.dbg.GetListener(), self.background_pid, error
        )
        self.assertTrue(error.Success() and process, PROCESS_IS_VALID)
        main_module = target.FindModule(lldb.SBFileSpec("a.out"))
        self.assertIsNotNone(main_module)
        self.assertFalse(
            main_module.IsFileBacked(),
            "The module should not be backed by a file on disk.",
        )

        error = process.Destroy()
        self.assertSuccess(
            error, "couldn't destroy process %s" % background_process.pid
        )

    @skipIfRemote
    def test_module_spec_list_indexing(self):
        """Test that SBModuleSpecList supports Pythonic indexing."""
        self.build()
        libfoo_path = self.getBuildArtifact("libfoo.a")
        specs = lldb.SBModuleSpecList.GetModuleSpecifications(libfoo_path)
        count = specs.GetSize()
        self.assertGreater(count, 0, "Archive should have at least one module spec")

        # Integer indexing: positive indices
        for i in range(count):
            self.assertEqual(
                str(specs[i]),
                str(specs.GetSpecAtIndex(i)),
                "specs[%d] should match GetSpecAtIndex(%d)" % (i, i),
            )

        # Integer indexing: negative indices
        self.assertEqual(
            str(specs[-1]),
            str(specs.GetSpecAtIndex(count - 1)),
            "specs[-1] should match last element",
        )
        self.assertEqual(
            str(specs[-count]),
            str(specs.GetSpecAtIndex(0)),
            "specs[-count] should match first element",
        )

        # Integer indexing: out of bounds raises IndexError
        self.assertRaises(IndexError, lambda: specs[count])
        self.assertRaises(IndexError, lambda: specs[-count - 1])

        # Unsupported key type raises TypeError
        self.assertRaises(TypeError, lambda: specs[1.5])

        # String indexing: lookup by file basename
        spec0 = specs.GetSpecAtIndex(0)
        basename = spec0.GetFileSpec().GetFilename()
        if basename:
            found = specs[basename]
            self.assertIsNotNone(found, "Should find spec by basename '%s'" % basename)
            self.assertEqual(
                found.GetFileSpec().GetFilename(),
                basename,
                "Found spec basename should match",
            )

        # String indexing: lookup by partial path (endswith matching)
        fullpath = str(spec0.GetFileSpec())
        if fullpath:
            found = specs[fullpath]
            self.assertIsNotNone(found, "Should find spec by full path '%s'" % fullpath)

        # String indexing: missing basename returns None
        self.assertIsNone(
            specs["nonexistent_file.xyz"],
            "Lookup of nonexistent basename should return None",
        )



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class PtrToArrayDataFormatterTestCase(TestBase):
    def test_with_run_command(self):
        """Test that LLDB handles the clang typeclass Paren correctly."""
        self.build()
        self.data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def data_formatter_commands(self):
        """Test that LLDB handles the clang typeclass Paren correctly."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format delete hex", check=False)
            self.runCmd("type summary clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.expect(
            "expression *(int (*)[3])foo",
            substrs=["(int[3]) $", "[0] = 1", "[1] = 2", "[2] = 3"],
        )

        self.expect(
            "expression *(int (*)[3])foo",
            matching=False,
            substrs=["01 00 00 00 02 00 00 00 03 00 00 00"],
        )
        self.expect(
            "expression *(int (*)[3])foo",
            matching=False,
            substrs=["0x000000030000000200000001"],
        )



"""
Test number of threads.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class ThreadExitTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers for our breakpoints.
        self.break_1 = line_number("main.cpp", "// Set first breakpoint here")
        self.break_2 = line_number("main.cpp", "// Set second breakpoint here")
        self.break_3 = line_number("main.cpp", "// Set third breakpoint here")
        self.break_4 = line_number("main.cpp", "// Set fourth breakpoint here")

    @skipIfWindows  # This is flakey on Windows: llvm.org/pr38373
    def test(self):
        """Test thread exit handling."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint with 1 location.
        bp1_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_1, num_expected_locations=1
        )
        bp2_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_2, num_expected_locations=1
        )
        bp3_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_3, num_expected_locations=1
        )
        bp4_id = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_4, num_expected_locations=1
        )

        # The breakpoint list should show 1 locations.
        self.expect(
            "breakpoint list -f",
            "Breakpoint location shown correctly",
            substrs=[
                "1: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.break_1,
                "2: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.break_2,
                "3: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.break_3,
                "4: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.break_4,
            ],
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)
        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        stopped_thread = lldbutil.get_one_thread_stopped_at_breakpoint_id(
            process, bp1_id
        )
        self.assertIsNotNone(stopped_thread, "Process is not stopped at breakpoint 1")

        # Get the number of threads
        num_threads = process.GetNumThreads()
        self.assertGreaterEqual(
            num_threads,
            2,
            "Number of expected threads and actual threads do not match at breakpoint 1.",
        )

        # Run to the second breakpoint
        self.runCmd("continue")
        stopped_thread = lldbutil.get_one_thread_stopped_at_breakpoint_id(
            process, bp2_id
        )
        self.assertIsNotNone(stopped_thread, "Process is not stopped at breakpoint 2")

        # Update the number of threads
        new_num_threads = process.GetNumThreads()
        self.assertEqual(
            new_num_threads,
            num_threads + 1,
            "Number of expected threads did not increase by 1 at bp 2.",
        )

        # Run to the third breakpoint
        self.runCmd("continue")
        stopped_thread = lldbutil.get_one_thread_stopped_at_breakpoint_id(
            process, bp3_id
        )
        self.assertIsNotNone(stopped_thread, "Process is not stopped at breakpoint 3")

        # Update the number of threads
        new_num_threads = process.GetNumThreads()
        self.assertEqual(
            new_num_threads,
            num_threads,
            "Number of expected threads is not equal to original number of threads at bp 3.",
        )

        # Run to the fourth breakpoint
        self.runCmd("continue")
        stopped_thread = lldbutil.get_one_thread_stopped_at_breakpoint_id(
            process, bp4_id
        )
        self.assertIsNotNone(stopped_thread, "Process is not stopped at breakpoint 4")

        # Update the number of threads
        new_num_threads = process.GetNumThreads()
        self.assertEqual(
            new_num_threads,
            num_threads - 1,
            "Number of expected threads did not decrease by 1 at bp 4.",
        )

        # Run to completion
        self.runCmd("continue")

        # At this point, the inferior process should have exited.
        self.assertState(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



"""
Testlldb Python SBFrame APIs IsInlined() and GetFunctionName().
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class InlinedFrameAPITestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to of function 'c'.
        self.source = "inlines.c"
        self.first_stop = line_number(
            self.source, "// This should correspond to the first break stop."
        )
        self.second_stop = line_number(
            self.source, "// This should correspond to the second break stop."
        )

    def test_stop_at_outer_inline(self):
        """Exercise SBFrame.IsInlined() and SBFrame.GetFunctionName()."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c by the name of 'inner_inline'.
        breakpoint = target.BreakpointCreateByName("inner_inline", "a.out")
        self.trace("breakpoint:", breakpoint)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() > 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        import lldbsuite.test.lldbutil as lldbutil

        stack_traces1 = lldbutil.print_stacktraces(process, string_buffer=True)
        if self.TraceOn():
            print(
                "Full stack traces when first stopped on the breakpoint 'inner_inline':"
            )
            print(stack_traces1)

        # The first breakpoint should correspond to an inlined call frame.
        # If it's an inlined call frame, expect to find, in the stack trace,
        # that there is a frame which corresponds to the following call site:
        #
        #     outer_inline (argc);
        #
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        frame0 = thread.GetFrameAtIndex(0)
        if frame0.IsInlined():
            filename = frame0.GetLineEntry().GetFileSpec().GetFilename()
            self.assertEqual(filename, self.source)
            self.expect(
                stack_traces1,
                "First stop at %s:%d" % (self.source, self.first_stop),
                exe=False,
                substrs=["%s:%d" % (self.source, self.first_stop)],
            )

            # Expect to break again for the second time.
            process.Continue()
            self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
            stack_traces2 = lldbutil.print_stacktraces(process, string_buffer=True)
            if self.TraceOn():
                print(
                    "Full stack traces when stopped on the breakpoint 'inner_inline' for the second time:"
                )
                print(stack_traces2)
                self.expect(
                    stack_traces2,
                    "Second stop at %s:%d" % (self.source, self.second_stop),
                    exe=False,
                    substrs=["%s:%d" % (self.source, self.second_stop)],
                )



"""
Add a test to verify our test instance returns something non-None for
an id(). Other parts of the test running infrastructure are counting on this.
"""

from lldbsuite.test.lldbtest import TestBase


class TestIdTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_id_exists(self):
        self.assertIsNotNone(self.id(), "Test instance should have an id()")



"""Helper library to traverse data emitted for Rust enums """
from lldbsuite.test.lldbtest import *

DISCRIMINANT_MEMBER_NAME = "$discr$"
VALUE_MEMBER_NAME = "value"


class RustEnumValue:
    def __init__(self, value: lldb.SBValue):
        self.value = value

    def getAllVariantTypes(self):
        result = []
        for i in range(self._inner().GetNumChildren()):
            result.append(self.getVariantByIndex(i).GetDisplayTypeName())
        return result

    def _inner(self) -> lldb.SBValue:
        return self.value.GetChildAtIndex(0)

    def getVariantByIndex(self, index):
        return (
            self._inner()
            .GetChildAtIndex(index)
            .GetChildMemberWithName(VALUE_MEMBER_NAME)
        )

    @staticmethod
    def _getDiscriminantValueAsUnsigned(discr_sbvalue: lldb.SBValue):
        byte_size = discr_sbvalue.GetType().GetByteSize()
        error = lldb.SBError()

        # when discriminant is u16 Clang emits 'unsigned char'
        # and LLDB seems to treat it as character type disalowing to call GetValueAsUnsigned
        if byte_size == 1:
            return discr_sbvalue.GetData().GetUnsignedInt8(error, 0)
        elif byte_size == 2:
            return discr_sbvalue.GetData().GetUnsignedInt16(error, 0)
        elif byte_size == 4:
            return discr_sbvalue.GetData().GetUnsignedInt32(error, 0)
        elif byte_size == 8:
            return discr_sbvalue.GetData().GetUnsignedInt64(error, 0)
        else:
            return discr_sbvalue.GetValueAsUnsigned()

    def getCurrentVariantIndex(self):
        default_index = 0
        for i in range(self._inner().GetNumChildren()):
            variant: lldb.SBValue = self._inner().GetChildAtIndex(i)
            discr = variant.GetChildMemberWithName(DISCRIMINANT_MEMBER_NAME)
            if discr.IsValid():
                discr_unsigned_value = RustEnumValue._getDiscriminantValueAsUnsigned(
                    discr
                )
                if variant.GetName() == f"$variant${discr_unsigned_value}":
                    return discr_unsigned_value
            else:
                default_index = i
        return default_index

    def getFields(self):
        result = []
        for i in range(self._inner().GetNumChildren()):
            type: lldb.SBType = self._inner().GetType()
            result.append(type.GetFieldAtIndex(i).GetName())
        return result

    def getCurrentValue(self) -> lldb.SBValue:
        return self.getVariantByIndex(self.getCurrentVariantIndex())
